/* ************************************************************************* */
/* cg_fe.h: Prototypes for the code-generation front end (cg_fe) functions   */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */



#ifndef __CG_FE_H__
#define __CG_FE_H__
/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <stdio.h>
#include "p_list.h"
#include "iopasm_types.h"
#include "i_table.h"
#include "iopasm_global.h"


/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */
#define ILLEGAL_OPERATOR (-1)

#define OPTION_ENDS (0)
#define OPT_DEFAULT_ON  (1)
#define OPT_DEFAULT_OFF (2)
#define OPT_ON (OPT_DEFAULT_ON)
#define OPT_OFF (OPT_DEFAULT_OFF)


/* ************************************************************************* */
/*                                PROTOTYPES                                 */
/* ************************************************************************* */
cg_context_t *cg_context_create(int arch,array_t *labels);
int cg_gen_fsm_code(cg_context_t *ctxt,inst_t *first,inst_t *last,int pass);
int cg_gen_seq_code(cg_context_t *ctxt,inst_t *inst,int pass,int scope);
int cg_cg_set_pc(cg_context_t *ctxt, int pc);
int cg_get_arch(cg_context_t *ctxt);
reg_table_t *cg_get_special_reg_table(cg_context_t *ctxt);
void cg_set_special_reg_table(cg_context_t *ctxt,
			      const reg_table_t *special_regs);
reg_table_t *cg_get_reg_table(cg_context_t *ctxt);
inst_table_t *cg_get_inst_table(cg_context_t *ctxt);
array_t *cg_get_fsm_symbols(cg_context_t *ctxt);
array_t *cg_get_label_array(cg_context_t *ctxt);
inst_table_t *cg_get_inst_table(cg_context_t *ctxt);
int cg_check_memory_region(cg_context_t *ctxt,int pc,int size);
int cg_set_memory_region(cg_context_t *ctxt,int pc,int size);
void cg_set_reg_table(cg_context_t *ctxt,const reg_table_t *reg_table);
void cg_set_inst_table(cg_context_t *ctxt,const inst_table_t *inst_table);
unsigned int cg_convert_seq_addr(cg_context_t *ctxt,unsigned int pc);
unsigned int cg_convert_fsm_addr(cg_context_t *ctxt,unsigned int pc);
void cg_set_seq_boundrary(cg_context_t *ctxt,int boundrary);
void cg_set_fsm_boundrary(cg_context_t *txt,int boundrary);
int cg_get_fsm_boundrary(cg_context_t *ctxt);
int cg_get_seq_boundrary(cg_context_t *ctxt);
int cg_get_fsm_size(cg_context_t *ctxt);
void cg_set_fsm_size(cg_context_t *ctxt,int size);
cg_operator_t *cg_get_be_operators(cg_context_t *context);
void cg_set_be_operators(cg_context_t *context,const cg_operator_t *operators);
int cg_get_memory_size(cg_context_t *ctxt);
char *cg_get_reg_table_type(cg_context_t *ctxt) ;
unsigned char *cg_get_code_buffer(cg_context_t *ctxt);
void cg_set_code_buffer(cg_context_t *ctxt,unsigned char *code_buffer);
int cg_get_row_len(cg_context_t *ctxt);
void cg_post_code_generation(cg_context_t *ctxt,plist_t *a_list);
char *cg_get_scope_name(cg_context_t *ctxt);
char **cg_get_cpp_macros(cg_context_t *ctxt);
void cg_set_cpp_macros(cg_context_t *ctxt,char **macros);
int cg_get_cpp_macro_count(cg_context_t *ctxt);
int cg_get_file_index_count(cg_context_t *ctxt);
int cg_index_file(cg_context_t *ctxt,char *file_name);
char *cg_get_file_by_index(cg_context_t *ctxt,int index);
int cg_get_file_index_by_name(cg_context_t *ctxt,char *name);
void cg_set_namespace(cg_context_t *ctxt,char *namespace);
char *cg_get_namespace(cg_context_t *ctxt);
int cg_eval_operator(cg_context_t *context,char *operator, int *meta_data,
		     int meta_len,int *buffer,int buffer_len,inst_t *inst,
		     int inum,arg_t *arg1,arg_t *arg2,int arg_no);
plist_t *cg_optimize_fsm_states(cg_context_t *ctxt,plist_t *a_list);
int cg_set_machine_option(cg_context_t *ctxt,char *machine_opt,int value);
int cg_get_machine_option(cg_context_t *ctxt,char *machine_opt,int *err);
int cg_check_machine_option(cg_context_t *ctxt,char *machine_opt,int wanted);
void cg_init_machine_options(cg_context_t *ctxt,char **options,int *opt_vals,int n_opts);
int cg_get_memory_bank_count(cg_context_t *ctxt);
void cg_set_memory_bank_count(cg_context_t *ctxt,int n_banks);
void cg_clear_memory_region(cg_context_t *ctxt,int pc,int size);
void cg_delete(cg_context_t *ctxt);
#endif /* end of cg_fe.h */
