/* ************************************************************************* */
/* global_rules.c : Architecture independent rules available for forming new */
/*                  rule chains in a back-end.                               */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */



/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "p_list.h"
#include "cg_fe.h"
#include "i_table.h"



/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */



/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */



/* ************************************************************************* */
/*                              CONSTANT VARIABLES                           */
/* ************************************************************************* */



/* ************************************************************************* */
/*                    PROTOTYPES FOR STATIC FUNCTIONS                        */
/* ************************************************************************* */



/* ************************************************************************* */
/* PROTOTYPE   : int arg2_range_check(cg_context_t *ctxt,inst_t *inst,       */
/*                                  int i_num,unsigned char *buf,            */
/* 	                            int buf_size);                           */
/*                                                                           */
/* ARGUMENTS   : (Please refer to the separate documentation on rule-        */
/*                functions)                                                 */
/*                                                                           */
/* FUNCTION    : Validate 2nd argument and generate warning if out of range  */
/*                                                                           */
/* RETURN VALUE: 1 if rule is ok, may exit if rule is violated               */
/* ************************************************************************* */
int arg2_range_check(cg_context_t *ctxt,inst_t *inst,int i_num,unsigned char *buf,
	       int buf_size)
{
/*   int t_val; */
/*   char *arg_val; */
/*   arg_t *an_arg; */
/*   assert ((ctxt != NULL) && (inst != NULL) &&(buf != NULL) && (buf_size > 0));   */

/*   an_arg = get_first_arg(inst); */
/*   an_arg = get_next_arg(an_arg); */

/*   if (get_arg_type(an_arg) == ARG_STACK) {  */
/*     convert_stack_arg(ctxt,inst,an_arg,FINAL_PASS); */
/*   } */
  
/*   arg_val = get_arg_val(an_arg); */

/*   t_val = atoi(arg_val); */
/*   if ((t_val < 0) || (t_val>31)) { */
/*     iopasm_err(ERR_IMM_OUT_OF_RANGE,inst,""); */
/*   } */
  
  return 1;
}



/* ************************************************************************* */
/* PROTOTYPE   : int validate_delay_slot(cg_context_t *ctxt,inst_t *inst,    */
/*                                       int i_num,unsigned char *buf,       */
/*                                       int buf_size);                      */
/*                                                                           */
/* ARGUMENTS   : (Please refer to the separate documentation on rule-        */
/*                functions)                                                 */
/*                                                                           */
/* FUNCTION    : Validate if a delay slot is used illegaly by an instruction */
/*                                                                           */
/* RETURN VALUE: 1 if rule is ok, may exit if rule is violated.              */
/* ************************************************************************* */
int validate_delay_slot(cg_context_t *ctxt,inst_t *inst,int i_num,
			  unsigned char *buf,int buf_size)
{
  inst_t *prev_inst;
  char *illegal_insts[] = { "ba", "bar", "bbc", "bbs", 
			    "bmi", "bnz", "bpl", "bz", NULL };
  int i;
  int inst_illegal = 0;
  char *inst_text,*prev_text;
  
  assert ((ctxt != NULL) && (inst != NULL) &&(buf != NULL) && (buf_size > 0));
  prev_inst = get_prev_inst(inst);

  if (prev_inst == NULL) return 1;

  inst_text = get_inst(inst);
  i = 0;

  while (illegal_insts[i] != NULL) {
    if (!strcmp(illegal_insts[i],inst_text)) {
      inst_illegal = 1;
    }
    i++;
  } /* end of while (illegal_insts[i] != NULL)) */
  
  if (inst != NULL) {
    if (get_inst_type(prev_inst) == INST_SEQ) { /* Check for delay slot */
      prev_text = get_inst(prev_inst);
      i = 0;
      while((illegal_insts[i] != NULL)) {
	if (!strcmp(illegal_insts[i],prev_text) && (inst_illegal)) {
	  /* We have a delay slot error */
	  fprintf(stderr,
		  "Warning: Possible incorrect use of delay-slot at line %d\n",
		  get_inst_line(prev_inst));
	}
	i++;
      }
    }
  }
  return 1;
}



/* ************************************************************************* */
/* PROTOTYPE   : int validate_dest_register(cg_context_t *ctxt,inst_t *inst, */
/*                                          int i_num,unsigned char *buf,    */
/*                			    int buf_size);                   */
/*                                                                           */
/* ARGUMENTS   : (Please refer to the separate documentation on rule-        */
/*                functions)                                                 */
/*                                                                           */
/* FUNCTION    : Validate if destination register is used in a proper manner */
/*                                                                           */
/* RETURN VALUE: 1 if rule is ok, may exit if rule is violated.              */
/* ************************************************************************* */
int validate_dest_register(cg_context_t *ctxt,inst_t *inst,int i_num,
				  unsigned char *buf,int buf_size)
{
  inst_t *next_inst,*this_inst;
  arg_t *this_arg;
  char *dest_arg;
  int i= 0;
  int next_arg_cnt;
  int this_arg_cnt;
  assert ((ctxt != NULL) && (inst != NULL) && 
	  (buf != NULL) && (buf_size > 0));

  if (get_prev_inst(inst) == NULL ) {
    return 1;
  }
  
  this_inst = get_prev_inst(inst);
  
  if (get_first_arg(this_inst) == NULL) { /* Inst does not have _any_ arguments */
    return 1;
  }

  else {
    this_arg = get_first_arg(this_inst);
    i = 1;
    
    while(get_next_arg(this_arg) != NULL) {
      this_arg = get_next_arg(this_arg);
      i++;
    }
    
    if ((get_arg_type(this_arg) == ARG_NORMAL_REG) ||
	(get_arg_type(this_arg) == ARG_SPECIAL_REG)) {
      
	/* Preserve and save information about destination reg */
      this_arg_cnt = i;
      dest_arg = get_arg_val(this_arg);
      next_inst = get_next_inst(this_inst);
      
      if (next_inst == NULL) {
	return 1;
      }
      
      this_arg = get_first_arg(next_inst);
      
      if (this_arg != NULL) {
	i = 1;
	
	/* Count number of arguments in instruction */
	while(get_next_arg(this_arg) != NULL) {
	  i++;
	  this_arg = get_next_arg(this_arg);
	}
      }
      
      else {
	i = 0;
      }
      
      next_arg_cnt = i; 
      i = 0;
      this_arg = get_first_arg(next_inst);

      do {
	if (((get_arg_type(this_arg) == ARG_NORMAL_REG) ||
	    (get_arg_type(this_arg) == ARG_SPECIAL_REG)) && 
	    (!strcmp(get_arg_val(this_arg),dest_arg)) && 
	    ((i==0) || ((i==1) && (next_arg_cnt > 1)))) { 
	  fprintf(stderr,
		  "Warning: Possible misuse of register %s in line %d (and line %d)\n",
		  dest_arg, get_inst_line(next_inst),get_inst_line(this_inst));
	}
      
	else {
	  return 1;
	}

	this_arg =get_next_arg(this_arg);

      } while(this_arg != NULL); 

    } /* End of if ((get_arg_type(this_arg) == ARG_NORMAL_REG) || */
  }   /* !if (get_first_arg(inst) == NULL) { */

  return 1;
}
