/* ************************************************************************* */
/* i_table.h: Prototypes for functions used to manipulate the instruction    */
/*           and register tables.                                            */
/*            instruction table entries.                                     */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */
#ifndef __I_TABLE_H__
#define __I_TABLE_H__



/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include "iopasm_global.h"
#include "iopasm_types.h"
#include "p_list.h"



/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */
#define SPECIAL_REG 0x1
#define REGULAR_REG 0x0 



/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */



/* ************************************************************************* */
/*                                PROTOTYPES                                 */
/* ************************************************************************* */
int get_inst_len(inst_table_t *i_table,int i_num);
int check_imm_val(inst_table_t *i_table,int i_num,unsigned int imm_val);
unsigned int get_src_regval(cg_context_t *ctxt,inst_t *inst,
			    int i_num,int reg_num);
unsigned int get_dst_regval(cg_context_t *ctxt,inst_t *inst,
			    int i_num,int reg_num);
unsigned int get_any_regval(cg_context_t *ctxt,inst_t *inst,	
			    int i_num,int reg_num);
unsigned int eval_src_immval(cg_context_t *ctxt,inst_t *inst,
			     int i_num,char *imm_val);
unsigned int eval_any_immval(cg_context_t *ctxt,inst_t *inst,
			     int i_num,char *imm_val);
unsigned int eval_dst_immval(cg_context_t *ctxt,inst_t *inst,
			     int i_num,char *imm_val);
int get_inst_number(inst_table_t *i_table,inst_table_t *row,inst_t *inst);
int get_reg_number(reg_table_t *reg_table,char *reg_name);
inst_table_t *create_itable_row(inst_t *inst);
void delete_itable_row(inst_table_t *row);
unsigned int get_opcode(inst_table_t *i_table,int i_num);
unsigned int generate_opcode(cg_context_t *ctxt,inst_table_t *i_table,inst_t *inst,
			     int i_num,int pass);
int reg_type_detect(reg_table_t *regs,reg_table_t *special_regs,
		    char *reg_name);
int sregs2regs(inst_table_t *row);
void save_register_macros(FILE *output_file,reg_table_t *registers,char *scope);
void save_inst_macros(FILE *output_file,inst_table_t *i_table,char *scope);
int xlate_reg2arch(reg_table_t *table,char *reg_name);
int xlate_reg2asm(reg_table_t *table,char *reg_name);
int get_field_size(inst_table_t *i_table,int i_num,int field_no);
#endif /* End of __I_TABLE_H__ */
/* end of i_table.h */
