/* ************************************************************************* */
/* i_table_types.h: Complete type definitions for types used in backends     */
/*                                                                           */
/*    Copyright 2002-2004 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */


#ifndef __I_TABLE_TYPES_H__
#define __I_TABLE_TYPES_H__
/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <stdlib.h>
#include "p_list.h"
#include "cg_fe.h"

/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */
#define MAX_ARGS 3 
#define MAX_FUNCS 8
#define TEMPLATE_LEN 4
#define I_BUFLEN 8
#define R ('R')   /* Register */
#define I ('I')   /* Immediate */
#define D ('D')   /* Data memory - not used */
#define C ('C')   /* Use Rule-Checking function */
#define X ('X')   /* 32-bit immediate */
#define S ('S')   /* Special register */

/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */
struct __inst_rule_t {
  char *rule_name;
  int gen_func;
  int (*rule_func[MAX_FUNCS])(cg_context_t *,inst_t *,int,unsigned char *,int);
};

typedef struct __inst_rule_t rule_set_t; 

struct __inst_table_t {
  char *inst_name;
  int num_args;
  char arg_type[MAX_ARGS];        /* Flags marking argument type */
  int arg_field[MAX_ARGS];       /* Argument field starts at bit ? */
  int field_len[MAX_ARGS];       /* Argument field is x bits long */
  int inst_len;                   /* Length of instruction in bytes */
  unsigned char i_template[4];    /* Instruction template */
  const rule_set_t *rule_set;     /* Rule set used for checking this inst */
};

struct __reg_table_t {
  char *reg_name;          /* Name of register as it appears in instructions */
  unsigned int reg_val;    /* Value associated with register (to form instruction) */
  int cg_reg_type;         /* Type of register - Special, Normal, Extended */
  int arch_reg_type;       /* Type of register seen from architectural view */
};

struct __mem_table_t {
  int inst_size;              /* Instruction size in bytes */
  int prog_mem_width;         /* The "witdh" of the program memory [bits] */
  int prog_mem_height;        /* The "height" of the program memory [rows] */
  int data_mem_width;         /* The "width" of data memory [bits] */
  int data_mem_height;        /* The "height" of data memory [rows] */
  unsigned int max_immediate; /* Max value of immediate */
  unsigned int max_reg_addr;  /* Max value of regaddr */
  unsigned int max_addr;      /* Max value of inst addr */
  char *prog_bitmap; /* Bitmap used to mark occupied locations in memory */
  char *data_bitmap; /*                    -- "" --                     */
  char *prog_memory; /* Pointer to the program memory (Instruction storage) */
  char *data_memory; /* Pointer to the data memory (Data storage (if any) */
};


#endif /* end of i_table_types.h */
