%{
  /* Lexical scanner for SPU SEQ/FSM assembler */

#include <stdio.h>
#include "iopasm.tab.h"
#include "p_list.h" 
#include "parser.h"
#include "cg_fe.h"
  /* External functions (defined in iopasm.y) */
  extern void add_fsm_timer();
  extern void adjust_line_no(int adjustment);
  extern void add_only_state();
  extern void add_fsm_option_inst(int option);
  static plist_t *i_list;
  static cg_context_t *context;
  void init_lexer(cg_context_t *ctxt,plist_t *a_list) { context = ctxt;  i_list = a_list; }
%}

%option caseless 8bit

%%

\n|@ { return NL; }
\:  { return ':'; }
\& { add_new_raw_arg(context,i_list,"&"); return '&'; }
\, { return ','; }
\* { add_new_raw_arg(context,i_list,"*"); return '*'; }
\/ { add_new_raw_arg(context,i_list,"/"); return '/'; }
\+ { add_new_raw_arg(context,i_list,"+"); return '+'; }
\- {  add_new_raw_arg(context,i_list,"-"); return '-'; }
\<\< { add_new_raw_arg(context,i_list,"<<"); return SHIFT_L; }
\>\> { add_new_raw_arg(context,i_list,">>"); return SHIFT_R; }
\=\= { add_new_raw_arg(context,i_list,"=="); return LOG_EQ; }
!\= {  add_new_raw_arg(context,i_list,"!="); return LOG_NEQ; }
! { add_new_raw_arg(context,i_list,"!"); return '!'; }
 { return ''; }
\^ { add_new_raw_arg(context,i_list,"^"); return '^'; }
\> { add_new_raw_arg(context,i_list,">"); return '>'; }
\< { add_new_raw_arg(context,i_list,"<"); return '<'; }
\<\> { add_new_raw_arg(context,i_list,"<>"); return NEQ; }
\( { add_new_raw_arg(context,i_list,"("); return '('; }
\) { add_new_raw_arg(context,i_list,")"); return ')'; }
\= { /* add_new_raw_arg(context,i_list,"="); */ return '='; }
\| { add_new_raw_arg(context,i_list,"|"); return '|'; }
~ { add_new_raw_arg(context,i_list,"~"); return '~'; }
p { return 'p'; }
\&\& { add_new_raw_arg(context,i_list,"&&"); return LOG_AND; }
\|\| { add_new_raw_arg(context,i_list,"&&"); return LOG_OR; }
\%[01]{1,32} {  return BIN_DIGIT; }
[0-9]+f { return FORWARD_LOCAL_LABEL; }
[0-9]+b { return BACKWARD_LOCAL_LABEL; }
[0-9]+\: {return LOCAL_ADDR_LABEL; }
[0-9]+ {  return DEC_DIGIT; }
("$"|0x)[0-9A-Fa-f]+ { return HEX_DIGIT; }
([01p\?]\_[01p\?]\_[01p\?]\_[01p\?]) { return OLD_FSM_SYMBOLISM; }
\.org { add_instruction(i_list,NULL,INST_ORIGIN,0,RF_ABSOLUTE); return ORG_OPERATOR; }
\.align { add_instruction(i_list,NULL,INST_ALIGN,0,RF_RELOCATE); return ALIGN_OPERATOR; }
\.fsm { return FSM_SWITCH; }
\.seq { return SEQ_SWITCH; }
\.dword { add_instruction(i_list,NULL,INST_DWORD,4,RF_RELOCATE); return DEF_DWORD; }
\.word { add_instruction(i_list,NULL,INST_DWORD,2,RF_RELOCATE);  return DEF_WORD; }
\.byte { add_instruction(i_list,NULL,INST_DWORD,1,RF_RELOCATE); return DEF_BYTE; }
\.end { return FILE_ENDS; }                                   
\. { return CURRENT_LOCATION; } 
\;.*|\#.*|\/\/.* { return COMMENT; }
only { add_instruction(i_list,NULL,INST_ONLY_STATE,0,RF_RELOCATE); return ONLY; }
keep { return KEEP; }
add|addq|and|andq|andqh|ba|bar|bbc|bbs { return SEQ_INST; }
addx|andx|di|ei|movex|orx|rwx|subx|xorx { return SEQ_INST; }
bnz|bmi|bpl|bz|fsm|fsmq|halt|jir|jnt|jsr|lsl|lslq|lsr|lsrq|lw { return SEQ_INST; }
move|moveh|movel|moveq|nop|not|or|orq { return SEQ_INST; }
ret|reti|rr|rrm|rrmh|rrmq|rrmx|rxor|rw|rwq  { return SEQ_INST; }
ssl|sslq|ssr|ssrq|sub|subq|sw|swx|swap|swsrq { return SEQ_INST; }
xor|xorq { return SEQ_INST; }

emask { add_instruction(i_list,NULL,INST_FSM_EVENT_MASK,0,RF_RELOCATE); return EMASK; }
umask { add_instruction(i_list,NULL,INST_FSM_UPDATE_MASK,0,RF_RELOCATE); return UMASK; }
reupdate { return REUPDATE; }
inp { add_instruction(i_list,NULL,INST_FSM_INP_FLAG,0,RF_RELOCATE); return INP; }
outp { add_instruction(i_list,NULL,INST_FSM_OUT_FLAG,0,RF_RELOCATE); return OUT; }
go_seq { return GO_SEQ; }
do_seq { return DO_SEQ; }
fsm_halt { return FSM_HALT; };
stb { add_instruction(i_list,NULL,INST_FSM_STB,0,RF_RELOCATE); return STB; }
inv_stb { return INV_STB; }
seq { return SEQ; }
timer { add_fsm_timer(); return TIMER; }
always { return ALWAYS; } 

e0|e1|e2|e3 { return REGISTER; }
e4|e5|e6|e7 { return REGISTER; }
e8|e9|e10|e11 { return REGISTER; }
e12|e13|e14|e15 { return REGISTER; }
e16 { return REGISTER; }

ecfg0|ecfg1|ecfg2|ecfg3 { return REGISTER; }
emask0|emask1|emask2|emask3 { return REGISTER; }
eval0|eval1|eval2|eval3 { return REGISTER; }
eaddr|immhi|jaddr { return REGISTER; }
r0|r1|r2|r3|r4|r5|r6|r7 { return REGISTER; }
r8|r9|r10|r11|r12|r13|r14|r15 { return REGISTER; }
r16|r17|r18|r19|r20|r21|r22|r23 { return REGISTER; }
r24|r25|r26|r27|r28|r29|r30|r31 { return REGISTER; }

i0|i1|i2|i3|i4|i5|i6|i7 { return REGISTER; }
i8|i9|i10|i11|i12|i13|i14|i15 { return REGISTER; }



p0|p1|p2|p3|p4|p5|p6|p7 { return REGISTER; }
p8|p9|p10|p11|p12|p13|p14|p15  { return  REGISTER; }
p16|p17|p18|p19|p20|p21|p22|p23 { return REGISTER; }
p24|p25|p26|p27|p28|p29|p30|p31 { return REGISTER; }

t0|t1|t2|t3 { return REGISTER; }

b0in|b1in { return REGISTER; }
b0out|b1out { return REGISTER; }
ctrl { return REGISTER; }
edge { return REGISTER; }
fsmpc { return REGISTER; }
fsm3_0|fsm7_4 { return REGISTER; }
gin { return REGISTER; }
gout { return REGISTER; }
index { return REGISTER; }
irp { return REGISTER; }
pc { return REGISTER; }
rega { return REGISTER; }
seqpc { return REGISTER; }
specs { return REGISTER; }
srp { return REGISTER; }
statin { return REGISTER; }
trigger { return REGISTER; }
wsts { return REGISTER; }

([a-zA-Z]+[0-9]*)+(_?[a-zA-Z0-9]+)* { return TEXT_LABEL; }
%%
