%{
/* ************************************************************************* */
/*    iopasm bison parser and associated helper functions.                   */
/*                                                                           */
/*    Copyright 2002-2004 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/*                                                                           */
/* Short revision history:                                                   */
/*                                                                           */
/*   Date      Description                                           Sign.   */
/*  =======================================================================  */
/*  2002-10-23 Module implemented                                    JohanJ  */
/*  2004-07-07 Module Cleaned up and partially rewritten             JohanJ  */
/*  2004-12-15 Comment headers added prior to release                JohanJ  */
/* ************************************************************************* */



/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "addr_calc.h"
#include "cg_fe.h"
#include "debug.h"
#include "dyn_array.h"
#include "dyn_stack.h"
#include "fsm_optimizer.h"
#include "iopasm_err.h"
#include "iopasm_global.h"
#include "labels.h"
#include "parser.h"
#include "p_list.h"


/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */
#define Y_STRLEN 16
#define SEQ_PARSER 1
#define FSM_PARSER 2
#define CBUFF_SIZE 256 
#define YYDEBUG 1
#define YYFPRINTF(YY_FILE,FORMAT,ARGS...) \
do {  char *tmp_string; \
  tmp_string = (parser_mode == SEQ_PARSER) ? "seq" :"fsm"; \
 fprintf(YY_FILE,"MODE=%s, yytext=%s:  ",tmp_string, yytext); \
 fprintf(YY_FILE,FORMAT,##ARGS); } \
while (0)

/* ************************************************************************* */
/*                      Prototypes for static variables                      */
/* ************************************************************************* */
  extern char *yytext;
  extern void init_lexer(cg_context_t *ctxt,plist_t *a_list);
  /* Uncomment the line below to turn on an outrageous amount of debug info */
  /* extern int yydebug = 1; */ 
  static int parser_mode = SEQ_PARSER;
  static plist_t *i_list;
  static int val_stack[128];
  static int stk_ptr = 0;
  static int file_ends = 0;
  static int op1;
  static int fsm_stage = 0;
  static unsigned int line_no = 1;
  static int timer_prio = 1;
  static cg_context_t *context;
  static int found_seq=0;
  static char *address_label;
  static int label_type = SCOPE_NO_SCOPE;
  static int ex_parser=0;
  static char *real_fn; /* Used to compare filenames in comment analysis */
  static int this_index;
 
/* ************************************************************************* */
/*                    Prototypes for static functions                        */
/* ************************************************************************* */
  static void stk_push(int);
  static int stk_pop();
  static void set_line_info(plist_t *a_list,int file_index,int line);
  static int chk_sym_type(char *sym);
  static unsigned int txt2dec(char *string);
  static unsigned int bin2dec(char *string);
  static unsigned int hex2dec(char *string);
  static char *imm2txt(unsigned int val);
  static int analyze_comment(char *comment,int line_no);
  static void check_parser_mode(plist_t *the_list,int mode,int expected);
  extern FILE *yyin;
  void yyerror (char const *s);
/*  extern void set_address_label(char *label); */
  %}


/* ************************************************************************* */
/*                    Define tokens and set error mode                       */
/* ************************************************************************* */
/* %debug */
%error-verbose

%token HEX_DIGIT COMMENT TEXT_LABEL SEQ_INST REGISTER DEC_DIGIT ADDR_LABEL 
%token BIN_DIGIT NEQ SHIFT_L SHIFT_R GEQ LEQ LOG_EQ BIT_EQ BIT_NEQ BIT 
%token LOG_NEQ NL FILE_ENDS FSM_SWITCH SEQ_SWITCH GO_SEQ DO_SEQ STB INV_STB 
%token INP OUT TIMER SEQ FSM_SYMDEF ALWAYS OLD_FSM_SYMBOLISM ORG_OPERATOR 
%token ONLY FSM_HALT ALIGN_OPERATOR EMASK UMASK REUPDATE DEF_DWORD DEF_WORD 
%token DEF_BYTE KEEP LOCAL_ADDR_LABEL FORWARD_LOCAL_LABEL 
%token BACKWARD_LOCAL_LABEL CURRENT_LOCATION


/* ************************************************************************* */
/*        Set precendence and priority for various arithmetic tokens         */
/* ************************************************************************* */
%left LOG_OR LOG_AND
%left '=' NEQ '<' '>' GEQ LEQ 
%left '+' '-'
%left '|' '&' '^' '!'
%left '*' '/' SHIFT_L SHIFT_R 
%left '~'
%left INVERT
%start GOAL
%%


/* ************************************************************************* */
/*                 Parser desription in context free grammar                 */
/* ************************************************************************* */
A_REGISTER: REGISTER { char *t_string; 
                       int reg_type;
                       t_string = duplicate_string(yytext);
                       reg_type = xlate_reg2asm(cg_get_reg_table(context),t_string);
		       push_arg(i_list,t_string,reg_type);
		       add_new_raw_arg(context,i_list,t_string);
		       IOPASM_FREE(t_string); };
  


ADDRESS_LABEL: A_LABEL ':' { 
                             if (parser_mode == SEQ_PARSER) {
                                 add_label(context,i_list,address_label,SCOPE_SEQ_ADDR);
				 set_line_info(i_list,this_index,line_no);
			       IOPASM_FREE(address_label);
                            } 
 
                            else {
			      found_seq = 0;
                              iopasm_verbose_msg("Warning: Timer priority implicitly set to 0\n");
			      timer_prio = 1; /* Set priority impliciltely to 1 */
			      if (fsm_stage) {
                              add_instruction(i_list,NULL,INST_STATE_END,0,RF_RELOCATE);
			      set_line_info(i_list,this_index,line_no);
			      }
			      add_label(context,i_list,address_label,SCOPE_FSM_ADDR);
			      set_line_info(i_list,this_index,line_no);
			      add_instruction(i_list,NULL,INST_STATE_BEGIN,0,RF_RELOCATE);
			      set_line_info(i_list,this_index,line_no);
                              fsm_stage = 1;
			      IOPASM_FREE(address_label);
                            }
                          } 

               |LOCAL_ADDR_LABEL { int op1=0; 
	                           int op2=0;
	                           char *t_string=NULL; 
		                   if (parser_mode == FSM_PARSER) {
				     iopasm_error(context,get_current_inst(i_list),
						  ERR_PARSER_EXTRA_MODE_SWITCH,
						  ".fsm specified but parser \
is already in FSM mode");
                                  }

	       else { 
                     op1 = (6+strlen(yytext))*sizeof(char);
	             t_string = (char *)IOPASM_MALLOC(op1) ;
	             assert (t_string !=NULL);
		     memset(t_string,0,op1);
		     t_string[0] = 'L';
		     op2 = strlen(yytext) -1; 
		     strncat(t_string,yytext,op2);
		     snprintf(&t_string[op2+1],op1-op2,"_%d",line_no);
		     add_label(context,i_list,t_string,SCOPE_SEQ_ADDR);
		     set_line_info(i_list,this_index,line_no);
		     IOPASM_FREE(t_string);

	            }
	       };
  

A_LABEL: TEXT_LABEL { address_label = duplicate_string(yytext);
                      label_type = ARG_LABEL; }

         | CURRENT_LOCATION { address_label = duplicate_string("."); 
	                      label_type = ARG_CURRENT_ADDR; }

         | FORWARD_LOCAL_LABEL { if (parser_mode == FSM_PARSER) {
	                           iopasm_error(context,get_current_inst(i_list),
						ERR_ILLEGAL_LOCAL_ADDR_LABEL,
						"Forward local address label (%s) \
not allowed in FSM mode",
						yytext);
				  } 
	                         op1 = (strlen(yytext)+1) * sizeof(char);
				 label_type = ARG_FW_LOCAL_LABEL;
	                         address_label = (char *) IOPASM_MALLOC(op1);
				 assert (address_label != NULL);
				 memset(address_label,0,op1);
				 address_label[0] = 'L';
				 strncat(&address_label[1],yytext,strlen(yytext)-1);
	                         }

         | BACKWARD_LOCAL_LABEL { if (parser_mode == FSM_PARSER) {
	                           iopasm_error(context,get_current_inst(i_list),
						ERR_ILLEGAL_LOCAL_ADDR_LABEL,
						"Backward local address label (%s) \
not allowed in FSM mode",
						yytext);
	                          }
	                          op1 = (strlen(yytext)+1) * sizeof(char);
				  label_type = ARG_BW_LOCAL_LABEL;
	                          address_label = (char *) IOPASM_MALLOC(op1);
				  assert(address_label != NULL);
				  memset(address_label,0,op1);
				  address_label[0] = 'L';
				  strncat(&address_label[1],yytext,strlen(yytext)-1);
	                        };


AN_IMMEDIATE: A_NUMBER { char *t_string=NULL; 
                         op1 = stk_pop(); 
                         t_string = imm2txt(op1);
			 add_new_raw_arg(context,i_list,t_string);
			 push_arg(i_list,t_string,ARG_IMMEDIATE);
		       };

A_NUMBER: HEX_DIGIT { int op1=0;
                      op1 = hex2dec(yytext); stk_push(op1); }
          |DEC_DIGIT { int op1=0;
                       op1 = txt2dec(yytext); stk_push(op1); }
          |BIN_DIGIT { int op1=0;
                       op1 = bin2dec(yytext); stk_push(op1); };

            
EXPRESSION: A_REGISTER 
	   | AN_IMMEDIATE
           | A_LABEL { inst_t *temp_inst;
                       push_arg(i_list,address_label,label_type);
		       temp_inst = get_current_inst(i_list);
		       set_reloc_flag(temp_inst,RF_RELOCATE);
		       add_new_raw_arg(context,i_list,yytext);
	             }

           | '~' EXPRESSION %prec INVERT { char *t_string=NULL;
                                           ex_parser = 1;
 	                                   t_string = duplicate_string("~");
					   assert(t_string != NULL);
					   push_arg(i_list,t_string,ARG_OPERATOR);
	                                 }

           | EXPRESSION '+' EXPRESSION { char *t_string=NULL;
                                         ex_parser = 1; 
	                                 t_string = duplicate_string("+");
				         assert(t_string != NULL);
	                                 push_arg(i_list,t_string,ARG_OPERATOR);
                                       }

           | EXPRESSION '-' EXPRESSION { char *t_string=NULL;
	                                 ex_parser = 1;
	                                 t_string = duplicate_string("-");
					 assert(t_string != NULL);
	                                 push_arg(i_list,t_string,ARG_OPERATOR); 
	                               }

          | EXPRESSION '*' EXPRESSION { char *t_string = NULL;
	                                ex_parser = 1;
					t_string = duplicate_string("*");
					assert(t_string != NULL);
					push_arg(i_list,t_string,ARG_OPERATOR); 
	                               }

          | EXPRESSION '/' EXPRESSION  { char *t_string=NULL;
	                                 ex_parser = 1;
	                                 t_string = duplicate_string("/");
					 assert(t_string != NULL);
	                                 push_arg(i_list,t_string,ARG_OPERATOR); 
	                               }

          | EXPRESSION SHIFT_L EXPRESSION { char *t_string=NULL;
	                                    ex_parser = 1;
	                                    t_string = duplicate_string("<<");
					    assert(t_string != NULL);
	                                    push_arg(i_list,t_string,ARG_OPERATOR); 
	                                  }

          | EXPRESSION SHIFT_R EXPRESSION { char *t_string=NULL;
	                                    ex_parser = 1;
	                                    t_string = duplicate_string(">>");
					    assert(t_string != NULL);
	                                    push_arg(i_list,t_string,ARG_OPERATOR); 
	                                  }

          |  EXPRESSION '^' EXPRESSION { char *t_string=NULL;
	                                 ex_parser = 1;
					 t_string = duplicate_string("^");
					 assert(t_string != NULL);
					 push_arg(i_list,t_string,ARG_OPERATOR); 
	                               }

          | EXPRESSION '|' EXPRESSION { char *t_string=NULL;
	                                ex_parser = 1;
					t_string = duplicate_string("|");
					assert(t_string != NULL);
					push_arg(i_list,t_string,ARG_OPERATOR); 
	                               }

          | EXPRESSION '&' EXPRESSION { char *t_string=NULL;
	                                ex_parser = 1;
					t_string = duplicate_string("&");
					assert(t_string != NULL);
					push_arg(i_list,t_string,ARG_OPERATOR); 
	                              }

          | EXPRESSION '<' EXPRESSION { char *t_string=NULL;
	                                ex_parser = 1;
					t_string = duplicate_string("<");
					assert(t_string != NULL);
					push_arg(i_list,t_string,ARG_OPERATOR); 
	                              }

          | EXPRESSION '>' EXPRESSION { char *t_string=NULL;
	                                ex_parser = 1;
					t_string = duplicate_string(">");
					assert(t_string != NULL);
					push_arg(i_list,t_string,ARG_OPERATOR); 
	                              }
  
          | '(' EXPRESSION ')' { ex_parser = 1; };




/* ************************************************************************* */
/* The goal is to get a complete assembly line. Here we define what is to be */
/* considered as a correct assembler line.                                   */
/* ************************************************************************* */
GOAL: ASMLINE NL { 
                   if (yytext[0] == '@') {
		     ;
		   }
		   else {
		     line_no++; 
		   }
		   return 0; 
                 }

      | NL { 

	     if (yytext[0] == '@') { 
	       ;
	     }
	     else {
	       line_no++;
	     }
	     return 0; };


ASMLINE: A_COMMENT 
         | ADDRESS_LABEL 
         | ADDRESS_LABEL ASM_INSTRUCTION 
         | ADDRESS_LABEL ASM_INSTRUCTION A_COMMENT
         | ADDRESS_LABEL A_COMMENT
         | ADDRESS_LABEL FSM_DIRECTIVES A_COMMENT
         | ADDRESS_LABEL FSM_DIRECTIVES
         | ADDRESS_LABEL FSM_TRANSITION A_COMMENT
         | ADDRESS_LABEL FSM_TRANSITION
         | ASM_INSTRUCTION
         | ASM_INSTRUCTION A_COMMENT
         | FSM_DIRECTIVES A_COMMENT
         | FSM_DIRECTIVES
         | FSM_TRANSITION A_COMMENT
         | FSM_TRANSITION;



ASM_INSTRUCTION: REAL_INSTRUCTION 
                 | PSEUDO_INSTRUCTION ;


/* ************************************************************************* */
/*                       Sequential instruction syntax                       */
/* ************************************************************************* */   
REAL_INSTRUCTION: SEQUENTIAL_INSTR EXPRESSION I_SEPARATOR EXPRESSION I_SEPARATOR EXPRESSION
                  | SEQUENTIAL_INSTR EXPRESSION I_SEPARATOR EXPRESSION 
                  | SEQUENTIAL_INSTR EXPRESSION 
                  | SEQUENTIAL_INSTR;

I_SEPARATOR: ',' { dyn_stack_t *tmp_stack;
                   arg_t *tmp_arg; 

                   tmp_stack = create_stack(delete_arg,
					    load_arg_object,
					    save_arg_object);
                   assert(tmp_stack != NULL);
                   tmp_arg = create_arg_stack(tmp_stack);
                   assert(tmp_arg != NULL);
                   add_arg(i_list,tmp_arg); 
                  };

SEQUENTIAL_INSTR: SEQ_INST {   
                                add_instruction(i_list,yytext,INST_SEQ,0,RF_RELOCATE); 
				set_line_info(i_list,this_index,line_no);
                           };

/* ************************************************************************* */
/*                     Comments and pseudo instructions                      */
/* ************************************************************************* */   
A_COMMENT: COMMENT { line_no = analyze_comment(yytext,line_no); };

PSEUDO_INSTRUCTION: ORIGIN
                    | ALIGN
                    | DWORD_INST
	            | FSM_SWITCH { fsm_stage = 0;
		                   if (parser_mode == FSM_PARSER) {
                                      add_instruction(i_list,NULL,INST_STATE_END,0,
					              RF_RELOCATE);
				      set_line_info(i_list,this_index,line_no);
                                   }
                                   parser_mode = FSM_PARSER; 
                                 }

                    | SEQ_SWITCH { fsm_stage = 0; 
                   	           if (parser_mode == FSM_PARSER) {
                                      add_instruction(i_list,NULL,INST_STATE_END,0,
					              RF_RELOCATE);
				      set_line_info(i_list,this_index,line_no);
				   }
                                   parser_mode = SEQ_PARSER; }

                    | FILE_ENDS { if (parser_mode == FSM_PARSER) {
 	                             add_instruction(i_list,NULL,INST_STATE_END,0,
				                     RF_RELOCATE);
				     set_line_info(i_list,this_index,line_no);
				    }
		                  file_ends = 1;
		                  return 0; }; 

ALIGN: ALIGN_OPERATOR EXPRESSION;
DWORD_INST: DEF_DWORD EXPRESSION;
ORIGIN: ORG_OPERATOR EXPRESSION;

/* ************************************************************************* */
/*                          FSM directives and flags                         */
/* ************************************************************************* */   
FSM_DIRECTIVES: FSM_FLAG
                | FSM_TIMER 
                | SEQ REAL_INSTRUCTION { found_seq = 1; }; 


/* Observe that the timer instruction is added by the lexer - we just 
 * need to do two things here:
 * 1. Check syntax
 * 2. Add operands, output conditions and the new_stage. 
 */

FSM_TIMER: TIMER TIMER_OPERAND ':' OUTPUT_CONDITION ':' NEW_STAGE; 

TIMER_OPERAND: KEEP { /* Keep timer - set flag & add priority */ 
                      char tmp_string[16];
		      dyn_stack_t *tmp_stack;
		      check_parser_mode(i_list,parser_mode,FSM_PARSER);
	              memset(tmp_string,0,16);
		      snprintf(tmp_string,16,"%d",timer_prio);
		      add_argument(context,i_list,NULL,ARG_TIMER_KEEP); 
		      add_argument(context,i_list,tmp_string,ARG_IMMEDIATE); 
		      tmp_stack = create_stack(delete_arg,
					       load_arg_object,
					       save_arg_object);
		      assert(tmp_stack != NULL);
                    }

               | EXPRESSION { char tmp_string[16];
	                      memset(tmp_string,0,16);
			      check_parser_mode(i_list,parser_mode,FSM_PARSER);
			      snprintf(tmp_string,16,"%d",timer_prio);
			      add_argument(context,i_list,tmp_string,ARG_IMMEDIATE);
	       };


FSM_FLAG: FSM_HALT { inst_t *temp_inst=NULL;
                     temp_inst = create_inst("",INST_FSM_HALT,0,RF_ABSOLUTE);
		     check_parser_mode(i_list,parser_mode,FSM_PARSER);
                     assert(temp_inst != NULL); 
                     add_inst(i_list,temp_inst);
		     set_line_info(i_list,this_index,line_no);
                   }

          |GO_SEQ { inst_t *temp_inst=NULL;
                    if (found_seq == 1) { 
		      return 1;
 	            } 
		    check_parser_mode(i_list,parser_mode,FSM_PARSER);
	            temp_inst = create_inst("",INST_FSM_GO_SEQ,0,RF_ABSOLUTE);
	            assert(temp_inst != NULL);
		    add_inst(i_list,temp_inst); 
		    set_line_info(i_list,this_index,line_no);
	          }

          |DO_SEQ { inst_t *temp_inst=NULL;
                    if (found_seq == 1) { 
		      return 1;
 	            } 
		    check_parser_mode(i_list,parser_mode,FSM_PARSER);
                    temp_inst = create_inst("",INST_FSM_DO_SEQ,0,RF_ABSOLUTE);
	            assert(temp_inst != NULL);
		    add_inst(i_list,temp_inst); 
		    set_line_info(i_list,this_index,line_no);
	          }

          |INV_STB { inst_t *temp_inst=NULL;
	             check_parser_mode(i_list,parser_mode,FSM_PARSER);
	             temp_inst = create_inst("",INST_FSM_INV_STB,0,RF_ABSOLUTE);
	             assert(temp_inst != NULL);
		     add_inst(i_list,temp_inst); 
		     set_line_info(i_list,this_index,line_no);
	           }

          |REUPDATE { inst_t *temp_inst=NULL;
	  		    check_parser_mode(i_list,parser_mode,FSM_PARSER);
                      temp_inst = create_inst("",INST_FSM_REUPDATE,0,RF_ABSOLUTE);
		      assert(temp_inst != NULL); 
		      add_inst(i_list,temp_inst);  
		      set_line_info(i_list,this_index,line_no);
	            }

          |STB '=' EXPRESSION { set_line_info(i_list,this_index,line_no); 
	                        check_parser_mode(i_list,parser_mode,FSM_PARSER); }
                                
          |EMASK '=' EXPRESSION { set_line_info(i_list,this_index,line_no);
                                  check_parser_mode(i_list,parser_mode,FSM_PARSER); }

          |UMASK '=' EXPRESSION { set_line_info(i_list,this_index,line_no); 
                                  check_parser_mode(i_list,parser_mode,FSM_PARSER); }

          |INP '=' EXPRESSION { set_line_info(i_list,this_index,line_no); 
                                check_parser_mode(i_list,parser_mode,FSM_PARSER); }
	                        
          |OUT '=' EXPRESSION { set_line_info(i_list,this_index,line_no); 
	                        check_parser_mode(i_list,parser_mode,FSM_PARSER); };
	  		        
      
FSM_TRANSITION: ALWAYS_STATEMENT 
                | ONLY_STATEMENT
                | INPUT_CONDITION ':' OUTPUT_CONDITION ':' NEW_STAGE;

ONLY_STATEMENT: ONLY ONLY_ARGUMENT { check_parser_mode(i_list,parser_mode,FSM_PARSER); 
                                   };

ONLY_ARGUMENT: EXPRESSION;


ALWAYS_STATEMENT: ALWAYS A_LABEL { check_parser_mode(i_list,parser_mode,FSM_PARSER);
                                   add_instruction(i_list,NULL,INST_STATE_XFER,0,RF_RELOCATE);
                                   add_argument(context,i_list,"?_?_?_?",ARG_FSM_IN);
                                   add_argument(context,i_list,"?_?_?_?",ARG_FSM_OUT);
                                   add_argument(context,i_list,address_label,
					        ARG_FSM_TRANSITION); 
				   IOPASM_FREE(address_label); 
                                };


INPUT_CONDITION: OLD_FSM_SYMBOLISM { check_parser_mode(i_list,parser_mode,FSM_PARSER);
                                     timer_prio = 0; /* Timer prio set to 0 */
                                     VERBOSE_MESSAGE(stderr,"Warning: Timer priority implicitly set to 0\n");
                                     if (chk_sym_type(yytext) == ARG_FSM_IN) {
				       add_instruction(i_list,NULL,INST_STATE_XFER,0,RF_RELOCATE);
				       set_line_info(i_list,this_index,line_no);
                                        add_argument(context,i_list,yytext,ARG_FSM_IN);
                                        }
 else return 1; };

OUTPUT_CONDITION: OLD_FSM_SYMBOLISM { check_parser_mode(i_list,parser_mode,FSM_PARSER);
                                      add_argument(context,i_list,yytext,ARG_FSM_OUT); };
                  
NEW_STAGE: EXPRESSION { check_parser_mode(i_list,parser_mode,FSM_PARSER); };
%%


/* ************************************************************************* */
/* PROTOTYPE   : void adjust_line_no(int adjustment);                        */
/*                                                                           */
/* ARGUMENTS   : "adjustment" is the adjustment to the current line number   */
/*                                                                           */
/* FUNCTION    : Adjusts the linenumber backwards or forwards.               */
/*                                                                           */
/* RETURN VALUE: None                                                        */
/* ************************************************************************* */
void adjust_line_no(int adjustment)
{
  line_no += adjustment;
}


/* ************************************************************************* */
/* PROTOTYPE   : void yyerror(char const *s);                                */
/*                                                                           */
/* ARGUMENTS   : "s" points to message string from the generated parser      */
/*                                                                           */
/* FUNCTION    : Report a simple error.                                      */
/*                                                                           */
/* RETURN VALUE: Never returns.                                              */
/* ************************************************************************* */
void yyerror(char const *s)
{
  inst_t *inst;
  inst = get_current_inst(i_list);
  iopasm_error(context,get_current_inst(i_list),ERR_SYNTAX_ERROR,
	       "Near symbol %s",s);
  exit(EXIT_FAILURE);
}


/* ************************************************************************* */
/* PROTOTYPE   : static int chk_sym_type(char *sym);                         */
/*                                                                           */
/* ARGUMENTS   : "*sym" points to a symbol string                            */
/*                                                                           */
/* FUNCTION    : Check if the symbol is output or input type.                */
/*                                                                           */
/* RETURN VALUE: Type of symbol.                                             */
/* ************************************************************************* */
static int chk_sym_type(char *sym) 
{
  assert(sym != NULL);
  if ((sym[0] == 'p')||(sym[2] == 'p')||(sym[4] == 'p')||(sym[6] == 'p')) {
    return ARG_FSM_OUT;
  }

  else {
    return ARG_FSM_IN;
  }
}


/* ************************************************************************* */
/* PROTOTYPE   : static void check_parser_mode(plist_t *the_list,int mode,   */
/*                                             int expected);                */
/*                                                                           */
/* ARGUMENTS   : "the_list" is a parsed list. "mode" is the current parser   */
/*               mode and "expected" is the mode we would expect.            */
/*                                                                           */
/* FUNCTION    : Check the parser mode and report an error if the mode is    */
/*               wrong.                                                      */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */ 
static void check_parser_mode(plist_t *the_list,int mode,int expected)
{
  inst_t *tmp_inst;
  char *tmp_string;
  tmp_string = (expected == FSM_PARSER) ? ".fsm":".seq";
  tmp_inst = get_current_inst(the_list);
  if (mode != expected) {
    iopasm_error(NULL,NULL,ERR_PARSER_MODE,"(%s is missing)",tmp_string);
  }
}


/* ************************************************************************* */
/* PROTOTYPE   : static unsigned int txt2dec(char *string)                   */
/*                                                                           */
/* ARGUMENTS   : "string" points to a string containing a number             */
/*                                                                           */
/* FUNCTION    : Convert a ASCII representation of a decimal number to its   */
/*               numerical equivalent.                                       */
/*                                                                           */
/* RETURN VALUE: the numerical representation.                               */
/* ************************************************************************* */
static unsigned int txt2dec(char *string)
{
  unsigned long temp_val;
  char *endptr[1];
  temp_val = strtoul(string,endptr,10);
  return  temp_val;
}


/* ************************************************************************* */
/* PROTOTYPE   : static unsigned int bin2dec(char *string)                   */
/*                                                                           */
/* ARGUMENTS   : "string" is a pointer to a string containing a binary digit */
/*                                                                           */
/* FUNCTION    : Converts a binary digit to its decimal representation       */
/*                                                                           */
/* RETURN VALUE: a number                                                    */
/* ************************************************************************* */
static unsigned int bin2dec(char *string)
{
  unsigned long temp_val;
  char *endptr[1];
  temp_val = strtoul(string+1,endptr,2);
  return temp_val;
}


/* ************************************************************************* */
/* PROTOTYPE   : static unsigned int hex2dec(char *string);                  */
/*                                                                           */
/* ARGUMENTS   : "string" is a pointer to a string.                          */
/*                                                                           */
/* FUNCTION    : convert a hexadecimal string to an unsigned int.            */
/*                                                                           */
/* RETURN VALUE: numerical value.                                            */
/* ************************************************************************* */
static unsigned int hex2dec(char *string)
{
  unsigned long temp_val;
  int n;
  char *endptr[1];
  n = (string[0] =='$') ? 1: 2;
  temp_val = strtoul(string+n,endptr,16);
  return temp_val;
}


/* ************************************************************************* */
/* PROTOTYPE   : static void stk_push(int val);                              */
/*                                                                           */
/* ARGUMENTS   : "val" is a value to place on the stack.                     */
/*                                                                           */
/* FUNCTION    : Push a value onto the stack.                                */
/*                                                                           */
/* RETURN VALUE: None                                                        */
/* ************************************************************************* */
static void stk_push(int val) { val_stack[stk_ptr] = val; stk_ptr++; }


/* ************************************************************************* */
/* PROTOTYPE   : static int stk_pop();                                       */
/*                                                                           */
/* ARGUMENTS   : none                                                        */
/*                                                                           */
/* FUNCTION    : Pop the internal parser stack                               */
/*                                                                           */
/* RETURN VALUE: Value popped from the stack.                                */
/* ************************************************************************* */
static int stk_pop() { return  val_stack[--stk_ptr]; }


/* ************************************************************************* */
/* PROTOTYPE   : static char *imm2txt(unsigned int val);                     */
/*                                                                           */
/* ARGUMENTS   : "val" is a value                                            */
/*                                                                           */
/* FUNCTION    : Convert a numerical value to text.                          */
/*                                                                           */
/* RETURN VALUE: pointer to a string containing the numerical value.         */
/* ************************************************************************* */
static char *imm2txt(unsigned int val)
{
  char temp_buffer[Y_STRLEN];
  char *ret_text;

  memset(temp_buffer,0,Y_STRLEN);
  snprintf(temp_buffer,Y_STRLEN,"%d",val);
  
  ret_text = (char *) duplicate_string(temp_buffer);
  assert(ret_text != NULL);

  return ret_text;
}


/* ************************************************************************* */
/* PROTOTYPE   : static void set_line_info(plist_t *a_list,int file_index,   */
/*                                         int line);                        */
/*                                                                           */
/* ARGUMENTS   : */
/*                                                                           */
/* FUNCTION    : */
/*                                                                           */
/* RETURN VALUE: */
/* ************************************************************************* */
static void set_line_info(plist_t *a_list,int file_index,int line)
{
  inst_t *current_inst;
  current_inst = get_current_inst(a_list);
  set_inst_line(current_inst,line);
  set_inst_file_index(current_inst,file_index);
}


/* ************************************************************************* */
/* PROTOTYPE   : static char *get_fname_part(char *file_path);               */
/*                                                                           */
/* ARGUMENTS   : "file_path" is a full filename and path.                    */
/*                                                                           */
/* FUNCTION    : Returns the file-name part of a /path/filename expression   */
/*                                                                           */
/* RETURN VALUE: Pointer to the filename                                     */
/* ************************************************************************* */
static char *get_fname_part(char *file_path)
{
  int i=0;

  assert(file_path != NULL);
  i = strlen(file_path);

  while((file_path[i] != '/') && (i>0)) {
    i--;
  }
  i = (i == 0) ? 0:i+1;

  return &file_path[i];

}


/* ************************************************************************* */
/* PROTOTYPE   : static int analyze_comment(char *comment,int line_no);      */
/*                                                                           */
/* ARGUMENTS   : "Comment" is a comment. line_no is the current line number  */
/*                                                                           */
/* FUNCTION    : Analyze the comment and find cpp style lin numbering and    */
/*               flags.                                                      */
/*                                                                           */
/* RETURN VALUE: Updated line number                                         */
/* ************************************************************************* */
static int analyze_comment(char *comment,int line_no)
{
  int i = 0;
  int j = 0;
  int line_adjust=0;
  int len;
  char *file = NULL;
  char *file_end = NULL;
  char *clean_name;
  char *found_name;
  char *flag_value = NULL;
  char esc_flag_value[] = { '4', 0 };

  len = strlen(comment);
  line_adjust = line_no;

  while (isspace(comment[i])) { 
    i++; 
  }
  if (comment[i] != '#') {
    line_adjust = line_no++;
  }

  else {
    i++;
    while (isspace(comment[i])) { 
      i++; 
    }
    if (isdigit(comment[i])) {
      line_adjust = atoi(&comment[i]);
    }

    while(isdigit(comment[i])||isspace(comment[i])) { 
      i++;
    }
    
    if (line_adjust == 0) {
      line_adjust = line_no;
    }
    
    else {
      i++;    
      file = &comment[i];
      file_end = file;
      j = 0;
      
      while((file_end[j] != '"') && (j<len)) { 
      j++; 
      };
      
      if (j == len) {
	return line_no;
      }

      flag_value = &file_end[j+1];
      i = j+1;
	
      }
      while (*flag_value ==' ') {
	flag_value++;
      }
      
      if (!isdigit(*flag_value)) {
	flag_value = esc_flag_value;
      }
      
      file_end[j] = '\0';
      j = strlen(file);
      /*       while((file[j] != '/') && (j>0)) {  */
      /* 	j--;  */
      /*       } /\* End of while((file[j] != '/') && (j>0)) *\/ */
      /* (j == 0) ? file: &file[j+1]; */
      found_name = get_fname_part(file);
      
      j = strlen(real_fn);
      while((real_fn[j] != '/') && (j>0)) {
	j--; 
      } /* End of while((file[j] != '/') && (j>0)) */
      clean_name = (j == 0) ? real_fn: &real_fn[j+1];
      
      switch(*flag_value) {
      case '1': /* 1 indicates start of new file */
	
	this_index = cg_index_file(context,found_name);
 	line_adjust--;  
	break;
	
      case '2': /* 2 indicates returning to a file */
	/* Find file index and update it */
	
	this_index = cg_get_file_index_by_name(context,found_name);
 	line_adjust--; 
	break;
	
      case '3': /* 3 From a system header */
	/* Do nothing for now... */
	break; 

	
      case '4': /* This is an artificial key. Ensure that we return to 
		 * the original file */
	  this_index == cg_index_file(context,found_name);
	  line_adjust++;
	  break;

      default: 
	line_adjust = line_no+1;
	break;
      } /* End of switch(*flag_value) */
  } /* End of !if (line_adjust == 0) */
return line_adjust;

}  


/* ************************************************************************* */
/* PROTOTYPE   : void add_fsm_timer();                                       */
/*                                                                           */
/* ARGUMENTS   : None                                                        */
/*                                                                           */
/* FUNCTION    : Add FSM timer pseudo instruction to the parse list.         */
/*                                                                           */
/* RETURN VALUE: none.                                                       */
/* ************************************************************************* */
void add_fsm_timer()
{
  inst_t *temp_inst;

  temp_inst = create_inst(NULL,INST_TIMER,0,RF_RELOCATE);
  assert(temp_inst != NULL);

  add_inst(i_list,temp_inst);
  set_inst_line(temp_inst,line_no);
}


/* ************************************************************************* */
/* PROTOTYPE   : int parse_file(plist_t *a_list,char *f_name,                */
/*                              cg_context_t *arch_context, char *src_fn,    */
/*                              int index);                                  */
/*                                                                           */
/* ARGUMENTS   : "a_list" is an empty parse list. "f_name" contains the file */
/*               name to open. "arch_context" is the architectural context.  */
/*               "src_fn" is the source-file name as specified on the cmd    */
/*               line. index is the file index number.                       */
/*                                                                           */
/* FUNCTION    : Start parsing of a file and monitor the progress            */
/*                                                                           */
/* RETURN VALUE: Return 1 on success. Otherwise iopasm_error(...) will be    */
/*               called.                                                     */
/* ************************************************************************* */
int parse_file(plist_t *a_list,char *f_name,cg_context_t *arch_context,
	       char *src_fn,int index)
{
  FILE *in_file;
  int ret_code;
  FILE *old_stdout,*new_stdout;


  in_file = fopen(f_name,"r");
  if (in_file == NULL) {
    iopasm_error(NULL,NULL,ERR_INPUT_FILE_MISSING,
		 "It is not possible to open %s",f_name);
  }

  this_index = cg_get_file_index_by_name(arch_context,get_fname_part(src_fn));
  if (this_index == -1) {
    this_index = cg_index_file(arch_context,get_fname_part(src_fn));
  }

  new_stdout = fopen("/dev/null","wt");
  assert(new_stdout != NULL);
  
  setvbuf(new_stdout,(char *) NULL,_IONBF,0);
  old_stdout = stdout;
  stdout = new_stdout;
  yyin = in_file;
  file_ends = 0;

  assert(arch_context != NULL);

  context = arch_context;
  this_index = index;
  real_fn = src_fn;
  i_list = a_list;
  init_lexer(context,a_list);
  line_no = 1;

  do {
    ret_code = yyparse();
    if (ret_code != 0) { /* We did not end due to end of file */
      stdout = old_stdout;
      fclose(new_stdout);
      iopasm_error(context,get_current_inst(a_list),ERR_SYNTAX_ERROR,NULL);
      exit(EXIT_FAILURE);
    }
    
  }  while((!file_ends)&&(!ret_code));
  
  resolve_labels(arch_context,a_list);
  stdout = old_stdout;
  fclose(new_stdout);
  return 1;
}
