/* ************************************************************************* */
/* iopasm_err.h : Error Handling and reporting prototypes and constants      */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */



#ifndef __IOPASM_ERR_H__
#define __IOPASM_ERR_H__
/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "iopasm_types.h"



/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */
extern void iopasm_err(cg_context_t *ctxt,int err,inst_t *inst);
/* ERRORS known by the I/O processor assembler */
#define ERR_UNSUPPORTED_ARCH     (101) 
#define ERR_FORK_FAILED          (102)
#define ERR_CPP_EXEC_FAILED      (103)
#define ERR_REPORT_BUG           (104)
#define ERR_CPP_STAGE_FAILED     (105)
#define ERR_MEMORY_REGION_USED   (106)
#define ERR_INVALID_CODE_SIZE    (107)
#define ERR_CANNOT_WRITE_FILE    (108)
#define ERR_SYNTAX_ERROR         (109)
#define ERR_UNDEFINED_LABEL      (110)
#define ERR_INPUT_FILE_MISSING   (111)
#define ERR_UNDEFINED_FW_LABEL   (112)
#define ERR_UNDEFINED_BW_LABEL   (113)
#define ERR_FILE_IS_EMPTY        (114)
#define ERR_ILLEGAL_WARN_FLAG    (115)
#define ERR_ILLEGAL_WARN_MODE    (116)
#define ERR_PARSER_MODE          (117)
#define ERR_ILLEGAL_LOCAL_ADDR_LABEL (118)
#define ERR_PARSER_EXTRA_MODE_SWITCH (119)
#define ERR_DUPLICATE_LABEL      (120)

/* Argument errors is in group 200 */
#define ERR_ARG_EXPECTED         (201)
#define ERR_UNEXPECTED_ARG       (202)
#define ERR_UNKNOWN_ARGUMENT     (203)
#define ERR_ILLEGAL_ARGUMENT     (204)
#define ERR_ARG_DWORD_EVAL       (205)
#define ERR_NON_CONSTANT_ARG     (206)

/* Immediate errors is in group 300 */
#define ERR_IMM_OUT_OF_RANGE     (301)
#define ERR_IMM_NOT_ALLOWED      (302)

/* FSM errors is in group 400 */
#define ERR_FSM_TOO_MANY_SEQS      (401)
#define ERR_FSM_TOO_MANY_TIMERS    (402)
#define ERR_FSM_TOO_MANY_XFERS     (403)
#define ERR_FSM_UNKNOWN_ERROR      (404)
#define ERR_FSM_BOUNDRARY          (405)
#define ERR_FSM_XFER_CONFLICT      (406)
#define ERR_FSM_TIMER_CONFLICT     (407)
#define ERR_FSM_ADDR_CONFLICT      (408)
#define ERR_FSM_NO_TIMER_ALLOWED   (409)
#define ERR_FSM_EMASK_IN_ONLY_XFER (410)
#define ERR_FSM_EMASK_WITH_GO_SEQ  (411)
#define ERR_FSM_NO_FSM_XFERS       (412)

/* Register errors is in group 500 */
#define ERR_UNKNOWN_REGISTER     (501)
#define ERR_REGISTER_NOT_ALLOWED (502)
#define ERR_REGISTER_NOT_IN_ARCH (503)
#define ERR_ILLEGAL_REGISTER_USE (504)
#define ERR_ILLEGAL_MASK_REG_USE (505)

/* Instruction errors is group 600*/
#define ERR_UNKNOWN_INSTRUCTION  (601)
#define ERR_SEQUENTIAL_BOUNDRARY (602)
#define ERR_ILLEGAL_INST_ORDER   (603)
#define ERR_ILLEGAL_MASK         (604)
#define ERR_32BIT_IMMEDIATE_IN_SEQ (605)
#define ERR_32BIT_UNALIGNED_IN_DS  (606)

/* File format errors is group 700 */
#define ERR_UNKNOWN_SYMFILE_FORMAT (701)
#define ERR_ARCHITECTURE_MISMATCH  (702)
#define ERR_INCOMPATIBLE_OBJ_FILE  (703)
#define ERR_NO_OBJECT_FILE_FOUND   (704)

/* Machine option errors & similar in group 800 */
#define ERR_UNKNOWN_MACHINE_OPTION (801)

/* Instruction warnings - Group 1000 */
#define WARN_UNALIGNED_64BIT_INST (1001)

/* Warnings - Rule function warnings in group 100*/
#define WARN_DELAY_SLOT_USAGE (100)
#define WARN_DEST_REG_USAGE   (101)
#define WARN_RETI_VIOLATION   (102)

/* Warnings - FSM warnings in group 200 */
#define WARN_STATE_TERMINATED (200)

/* Flags to manage soft error (serrors) behaviour (described by serror_t */
#define ERROR_OFF (0)
#define ERROR_ON (1)
#define WARNING_ON (2)

/* We assume that va_copy by itself is a macro, if it's there.  It's
   absent on e.g. gcc-2.95.  When absent, plain copy is assumed working.  */
#ifndef va_copy
#define va_copy(a,b) (a) = (b)
#endif

/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */



/* ************************************************************************* */
/*                              FUNCTION PROTOTYPES                          */
/* ************************************************************************* */
void iopasm_error(cg_context_t *ctxt,inst_t *inst,int err_no,
		  char *extra_fmt,... );
void iopasm_verbose_msg(char *fmt,...) ;
void set_error_mode(char *error_identifier,char *set_state);
#endif /* Matches #ifndef __IOPASM_ERR_H__ */
/* End of iopasm_err.h */
