/* ************************************************************************* */
/* iopasm_global.h : Global defines for IOP assembler                        */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */




#ifndef __IOPASM_GLOBAL_H__
#define __IOPASM_GLOBAL_H__
/* ************************************************************************* */
/*                                  DEFINITIONS                              */
/* ************************************************************************* */
/* TRUE and FALSE */
#define FALSE (0)
#define TRUE (!FALSE)

/* Code generation phases */
#define INTERMEDIATE_PASS (1) /* Intermediate code-generation pass           */
#define FINAL_PASS        (2) /* Final-code generation pass                  */

/* Code generation scopes */
#define SCOPE_FSM (4) 
#define SCOPE_SEQ (3)
/* Version data */
#define IOPASM_MAJOR_VERSION 1 /* Major version of IOPASM                    */
#define IOPASM_MINOR_VERSION 4 /* Minor version of IOPASM                    */

/* Miscellaneous defines */
#define MAX_STRLEN 255
#define MAX_BUFFER_SIZE (MAX_STRLEN)
#define OPT_STRING  0x01 /* Options should be interpreted as strings         */
#define OPT_INTEGER 0x02 /* Options should be interpreted as integers        */

#define ARCH_MPU    0x01 /* Architecture is MPU - set on command line        */
#define ARCH_SPU    0x02 /* Architecture is SPU - set on command line        */

#define CONTEXT_SEQ 0x01 /* code-generation context is seq (MPU or SPU)      */
#define CONTEXT_FSM 0x02 /* code-generation context is fsm (SPU)             */

#ifdef MEMORY_DEBUG
#define IOPASM_MALLOC(X) iopasm_malloc(__FILE__,__LINE__,X)
#define IOPASM_FREE(X) iopasm_free(__FILE__,__LINE__,X)
#else
#define IOPASM_MALLOC(X) malloc(X)
#define IOPASM_FREE(X) free(X)
#endif



#define IOPASM_INFO_MESSAGE(MESSG) fprintf(stderr,"iopasm informational: %s\n",MESSG); 
#define VERBOSE_MESSAGE(STREAM,FORMAT, ARGS...) \
if (check_option("verbose",(void *) 1,OPT_INTEGER)) { \
  fprintf(STREAM,FORMAT,##ARGS); }

/* ************************************************************************* */
/*                              FUNCTION PROTOTYPES                          */
/* ************************************************************************* */
void *get_option(char * option);
int check_option(char *option,void *option_val,int interpretation);
int str_compare(void *s1,void *s2);
void str_destroy(void *s1);

void iopasm_free(char *file,int line,void *data);
void *iopasm_malloc(char *file,int line,size_t size);
#endif /* End of iopasm_global.h */
