/* ************************************************************************* */
/* label.h: Abstraction of simple symbolic value used by code generators     */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */



#ifndef __LABELS_H__
#define __LABELS_H__
/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <stdio.h>
#include "iopasm_types.h"

/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */
#define SCOPE_NO_SCOPE  (0) /* Scope is not known, often used for tmp labels */
#define SCOPE_FSM_ADDR  (1) /* Label is an FSM address                       */
#define SCOPE_SEQ_ADDR  (2) /* Label is a sequential address                 */
#define SCOPE_SEQ_LOCAL_ADDR (SCOPE_SEQ_ADDR | 0x4) /* Now used as flags     */

/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */




/* ************************************************************************* */
/*                               PROTOTYPES                                  */
/* ************************************************************************* */
label_t *create_label(char *label_name,unsigned int label_value,int scope);
int change_label_value(label_t *label,unsigned int val);
unsigned int get_label_value(cg_context_t *context,label_t *label);
int delete_label(label_t *label);
int compare_labels(void *label1,void *label2);
/* unsigned int get_label_value(label_t *label); */
char *get_label_name(label_t *label);
int write_label_as_symbols(void *context,void *dst,void *elem);
int write_label_as_header(void *context,void *dst,void *elem);
int write_label_as_object(void *context,void *dst,void *elem);
void label_destructor(void *data);
label_t *load_label_object(FILE *input_file);
void dump_label_object(FILE *input_file);
int print_label(cg_context_t *ctxt,void *dst,void *elem);
int print_label_stdout(cg_context_t *ctxt,void *dst,void *elem);
#endif /* labels.h */
