/* ************************************************************************* */
/* load_operations.c : Load operations for symbols and objects               */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/* ************************************************************************* */



/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "iopasm_global.h"
#include "cg_fe.h"
#include "p_list.h"
#include "dyn_array.h"
#include "labels.h"
#include "meta_object.h"


/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */
#define MAX_STRLEN 255



/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */




/* ************************************************************************* */
/*                              CONSTANT VARIABLES                           */
/* ************************************************************************* */



/* ************************************************************************* */
/*                    PROTOTYPES FOR STATIC FUNCTIONS                        */
/* ************************************************************************* */
static void dump_filename_object(FILE *input_file,meta_object_t *load_object);


/* ************************************************************************* */
/* PROTOTYPE   : void load_symbol_file(array_t *labels,char *file_name);     */
/*                                                                           */
/* ARGUMENTS   : "labels" is an array of labels. "file_name" is the file to  */
/*               load the symbols from.                                      */
/*                                                                           */
/* FUNCTION    : Loads and defines a number of symbols (labels) from a       */
/*               previously saved symbol file.                               */
/*                                                                           */
/* RETURN VALUE: None                                                        */
/* ************************************************************************* */
void load_symbol_file(array_t *labels,char *file_name)  /* Defunct as per 2003-08-27 */
{                                                       /* Does not take label scope into */
  FILE *in_file;                                        /* consideration */
  label_t *a_label;
  char data[MAX_STRLEN];
  unsigned int label_val;
  
  int i;
  assert ((labels != NULL) && (file_name != NULL));
  
  in_file = fopen(file_name,"rt");
  assert(in_file != NULL);

  while (!feof(in_file)) {
   
    memset(data,0,MAX_STRLEN);
    if (fgets(data,MAX_STRLEN,in_file) != NULL) {
      i=0;

      while(data[i] != ' ') i++;
      data[i++] = '\0';

      while(data[i] == ' ') i++;

      label_val = strtoul(&data[i],NULL,16);
      a_label = create_label(data,label_val,SCOPE_NO_SCOPE);
      assert(a_label != NULL);
      add_array_elem(labels,(void *)a_label);
    } /* if (fgets(data,MAX_STRLEN,in_file) != NULL) */
  } /* end of while (!feof(in_file)) */
}


/* ************************************************************************* */
/* PROTOTYPE   : void load_object_file(cg_context_t *ctxt,plist_t *list,     */
/*                                     char *file_name);                     */
/*                                                                           */
/* ARGUMENTS   : "ctxt" is a valid context. "list" is a valid parse-list and */
/*               "file_name" is the name of a file.                          */
/*                                                                           */
/* FUNCTION    : Loads an object file from "file_name" into "list without    */
/*               parsing it.                                                 */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void load_object_file(cg_context_t *ctxt,plist_t *list,char *file_name)
{
  FILE *input_file;
  meta_object_t *load_object;
  inst_t *an_inst;
  arg_t *an_arg;
  label_t *a_label;
  array_t *label_array;
  unsigned int obj_count;
  unsigned int old_fname_index;
  unsigned int temp;
  char *fname;

  assert ((ctxt != NULL) && (list != NULL) && (file_name != NULL));
  label_array = cg_get_label_array(ctxt);

  input_file = fopen(file_name,"rb");
  assert (input_file != NULL);

  obj_count = chk_object_header(file_name,input_file,ctxt);
  old_fname_index = cg_get_file_index_count(ctxt);
  while(obj_count) {
    
    load_object = load_meta_object(input_file);
    obj_count--;

    switch (get_object_type(load_object)) {

    case SRC_FILENAME_ELEM:
      fname = load_obj_string(input_file,load_object);
      cg_index_file(ctxt,file_name);
      break;
      
      
    case INST_LIST_ELEM:
      an_inst = load_inst_object(input_file,load_object);
      delete_meta_object(load_object);
      temp = get_inst_file_index(an_inst);
      set_inst_file_index(an_inst,temp + old_fname_index);
      add_inst(list,an_inst);
      break;
      
    case ARG_LIST_ELEM:
      an_arg = (arg_t *) load_arg_object(input_file);
      assert(an_arg != NULL);
      delete_meta_object(load_object);
      add_arg(list,an_arg);
      break;
      
    case LABEL_ELEM:
      a_label = load_label_object(input_file);
      delete_meta_object(load_object);
      add_array_elem(label_array,(void *)a_label);
      break;

    default:
      dump_meta_object(load_object);
      break;
      
    } /* End of switch(...) */
  } /* End of while(!feof(input_file)) */
}

/* ************************************************************************* */
/* PROTOTYPE   : static void dump_filename_object(FILE *input_file,          */
/*                                                meta_object_t *load_object)*/
/*                                                                           */
/* ARGUMENTS   : "input_file" points to a valid input file. "meta_object"    */
/*               is the object descriptor object (hence meta object)         */
/*                                                                           */
/* FUNCTION    : Dump a filename object to stdout. Only used for debugging   */
/*               purposes.                                                   */
/*                                                                           */
/* RETURN VALUE: None                                                        */
/* ************************************************************************* */
static void dump_filename_object(FILE *input_file,meta_object_t *load_object)
{
  int len;
  char *string;
  assert ((input_file != NULL) && (load_object != NULL));
  len = get_object_size(load_object);

  string = (char *) IOPASM_MALLOC(sizeof(char)*(len+1));
  assert (string != NULL);
  
  memset(string,0,len+1);
  assert(fread(string,len,sizeof(char),input_file) == 1);
  fprintf(stdout,"     Object value      : %s\n",string);
  fprintf(stdout,"--------------------------------------------------------\n");
  IOPASM_FREE(string);
}



/* ************************************************************************* */
/* PROTOTYPE   : void dump_object_file(char *file_name,cg_context_t *ctxt);  */
/*                                                                           */
/* ARGUMENTS   : "file_name" is the name of a file. "ctxt" is a valid context*/
/*                                                                           */
/* FUNCTION    : Dump the contents of a object file onto stdout in textual   */
/*               form.                                                       */
/*                                                                           */
/* RETURN VALUE: Used for debugging.                                         */
/* ************************************************************************* */
void dump_object_file(char *file_name,cg_context_t *ctxt)
{
  FILE *input_file;
  meta_object_t *load_object;
  unsigned int obj_count,obj_num=0;

  assert ((ctxt != NULL) && (file_name != NULL));

  input_file = fopen(file_name,"rb");
  assert (input_file != NULL);

  obj_count = dump_object_header(input_file,ctxt);

  while(obj_count) {
    obj_num++;
    load_object = load_meta_object(input_file);
    obj_count--;

    switch (get_object_type(load_object)) {
      
    case SRC_FILENAME_ELEM:
      fprintf(stdout,"Dumping object of type %d (Source filename)\n",INST_LIST_ELEM);
      fprintf(stdout,"     Object size : %d\n",get_object_size(load_object));
      dump_filename_object(input_file,load_object);
      break;

    case INST_LIST_ELEM:
      fprintf(stdout,"Dumping object of type %d (Instruction)\n",INST_LIST_ELEM);
      fprintf(stdout,"     Object size : %d\n",get_object_size(load_object));
      dump_inst_object(input_file,load_object);
      delete_meta_object(load_object);
      break;
      
    case ARG_LIST_ELEM:
      fprintf(stdout,"Dumping object of type %d (Argument)\n",INST_LIST_ELEM);
      fprintf(stdout,"     Object size : %d\n",get_object_size(load_object));
      dump_arg_object(input_file,load_object);
      delete_meta_object(load_object);
      break;
      
    case LABEL_ELEM:
      fprintf(stdout,"Dumping object of type %d (Label)\n",INST_LIST_ELEM);
      fprintf(stdout,"     Object size : %d\n",get_object_size(load_object));
      dump_label_object(input_file);
      delete_meta_object(load_object);
      break;
      
    } /* End of switch(...) */
  } /* End of while(!feof(input_file)) */
}
	  
