/* ************************************************************************* */
/* p_list.h: Parser list (i-tree) header file                                */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/*                                                                           */
/* Short revision history:                                                   */
/*                                                                           */
/*   Date      Description                                          Sign.    */
/*  =======================================================================  */
/*  2002-10-22 Module implemented                                    JohanJ  */
/*  2003-01-18 Added get_inst_line(...) to return source-code line   JohanJ  */
/* ************************************************************************* */

#ifndef __P_LIST_H__
#define __P_LIST_H__
/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <stdio.h>
#include "iopasm_types.h"

/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */

/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */
#define INST_UNKNOWN       0x00  /* Instruction type unknown                 */
#define INST_SEQ           0x01  /* Sequential instruction                   */
#define INST_ORIGIN        0x02  /* Origin                                   */
#define INST_STATE_BEGIN   0x03  /* Marks the beginning of a state           */
#define INST_STATE_END     0x04  /* Marks the end of a state                 */
#define INST_STATE_XFER    0x05  /* State transition                         */
#define INST_LABEL         0x06  /* Instruction is a label (pseudo)          */
#define INST_TIMER         0x07  /* Timer instruction found in FSM mode      */
#define INST_FSM_OPTIONS   0x08  /* All state-options are arguments on this  */
#define INST_ONLY_STATE    0x09  /* Only possible state transfer             */
#define INST_ALIGN         0x0a  /* Alignment operator                       */
#define INST_DWORD         0x0b  /* Dword definition                         */
#define INST_WORD          0x0c  /* Word definition                          */
#define INST_BYTE          0x0d  /* Byte definition                          */

#define INST_FSM_INP_FLAG    0x0e 
#define INST_FSM_OUT_FLAG    0x0f
#define INST_FSM_DO_SEQ      0x11
#define INST_FSM_GO_SEQ      0x12 
#define INST_FSM_INV_STB     0x13
#define INST_FSM_STB         0x14
#define INST_FSM_HALT        0x15
#define INST_FSM_EVENT_MASK  0x16
#define INST_FSM_UPDATE_MASK 0x17
#define INST_FSM_REUPDATE    0x18

#define ARG_NO_ARG         0x00  /* Not an argument                          */
#define ARG_IMMEDIATE      0x01  /* Argument is an immediate (32-bit value)  */
#define ARG_NORMAL_REG     0x02  /* Argument is a register                   */
#define ARG_LABEL          0x04  /* Argument is a label                      */
#define ARG_FW_LOCAL_LABEL 0x05  /* Local label w/ forward reference         */
#define ARG_BW_LOCAL_LABEL 0x06  /* Local label w/ backward reference        */
#define ARG_FSM_IN         0x07  /* Argument specifies FSM INPUT (fsm)       */
#define ARG_FSM_OUT        0x08  /* Argument specifies FSM OUTPUT (fsm)      */
#define ARG_FSM_TRANSITION 0x09  /* Specifies the transition (fsm-transition)*/
#define ARG_FSM_MODIFIER   0x0a  /* Signal modifier for FSM combined         */
#define ARG_FSM_TIMER      0x0b /* FSM timer statement                      */
#define ARG_SPECIAL_REG    0x0c /* Special Register                         */
#define ARG_FSM_ONLY       0x0d /* only fsm transition in current state     */
#define ARG_STACK          0x0e /* Argument should be treated as a stack    */
#define ARG_OPERATOR       0x0f /* Argument holds an operator               */
#define ARG_TIMER_KEEP     0x10 /* Keep timer value                         */
#define ARG_CURRENT_ADDR   0x11 /* Current PC value */

#define RF_NO_CHANGE 0x00 /* No changes to relocation_information */
#define RF_RELOCATE  0x01 /* Instruction address is relocateable */
#define RF_ABSOLUTE  0x02 /* Instruction address is absolute */


/* ************************************************************************* */
/*                                PROTOTYPES                                 */
/* ************************************************************************* */
plist_t *create_plist();

inst_t *create_inst(char *inst,int i_type,unsigned int addr,int reloc_info); 
inst_t *get_current_inst(plist_t *list);
inst_t *get_first_inst(plist_t *list);
char *get_inst(inst_t *inst);
void set_inst_addr(inst_t *inst,unsigned int addr);
unsigned int get_inst_addr(inst_t *inst);
int get_inst_size(inst_t *inst);
int get_inst_reloc(inst_t *inst);
int add_inst(plist_t *list, inst_t *inst);
int add_origin(plist_t *list, inst_t *inst);

arg_t *create_arg(char *arg_val,int arg_len,int arg_type);
arg_t *create_arg_stack(dyn_stack_t *a_stack);
int  add_arg(plist_t *list,arg_t *arg);
arg_t *get_first_arg(inst_t *inst);
int get_arg_count(inst_t *inst);
arg_t *get_next_arg(arg_t *arg);
int add_inst_arg(inst_t *inst,arg_t *arg);
int get_arg_type(arg_t *arg);
void set_arg_type(arg_t *arg,int arg_type);
char get_arg_len(arg_t *arg);
char *get_arg_val(arg_t *arg);
dyn_stack_t *get_arg_stack(arg_t *arg);
void set_arg_val(arg_t *arg,char *arg_val,int arg_len);
inst_t *move_next_inst(inst_t *inst,inst_t *new_next);
inst_t *get_prev_inst(inst_t *inst);
inst_t *get_next_inst(inst_t *inst);
void set_next_inst(inst_t *where,inst_t *next);
void set_prev_inst(inst_t *where,inst_t *prev);
int get_inst_type(inst_t *inst);
void set_inst_size(inst_t *inst,int size);
unsigned int get_inst_line(inst_t *inst);
void set_inst_line(inst_t *inst,unsigned int line_no);
unsigned char *get_inst_code(inst_t *inst);
void set_inst_code(inst_t *inst,unsigned char *code,int code_len);
void delete_plist(plist_t *a_list);
int save_inst_object(FILE *out_file,inst_t *inst);
int save_arg_object(FILE *out_file,void *arg);
inst_t *load_inst_object(FILE *in_file,void *object);
void *load_arg_object(FILE *in_file); 
void dump_inst_object(FILE *in_file,meta_object_t *object);
void dump_arg_object(FILE *in_file,meta_object_t  *object);
void set_inst_file_index(inst_t *inst,int file_index);
int get_inst_file_index(inst_t *inst);
void delete_arg(void *the_arg);
void print_arg_val(void *arg);
int check_reloc_flag(inst_t *inst,int flag);
int clear_reloc_flag(inst_t *inst,int flag);
int set_reloc_flag(inst_t *inst,int flag);
int get_reloc_flag(inst_t *inst);
int add_raw_arg(arg_t *arg,char *string);
char *get_raw_arg(arg_t *arg);
void set_raw_arg(arg_t *arg,char *arg_val);
arg_t *get_current_arg(inst_t *inst);
arg_t *copy_arg(arg_t *arg);
inst_t *copy_inst(inst_t *an_inst);
int get_inst_count(plist_t *a_list,int inst_type);
int get_list_count(plist_t *a_list);
inst_t *synth_org_inst(unsigned int addr);
inst_t *synth_align_inst(unsigned int bit_val);
void insert_inst(inst_t *prior_to,inst_t *insert_this);
void delete_code(plist_t *a_list);
#endif /* end of template.h */
