/* ************************************************************************* */
/* parser.c : Helper functions called from FLEX and BISON generated parsers  */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/*                                                                           */
/* Short revision history:                                                   */
/*                                                                           */
/*   Date      Description                                          Sign.    */
/*  =======================================================================  */
/*  2004-07-07 Code moved during major clean up                     JohanJ   */
/* ************************************************************************* */



/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "cg_fe.h"
#include "dyn_array.h"
#include "dyn_stack.h"
#include "iopasm_err.h"
#include "labels.h"
#include "p_list.h"



/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */


/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */



/* ************************************************************************* */
/*                              CONSTANT VARIABLES                           */
/* ************************************************************************* */



/* ************************************************************************* */
/*                    PROTOTYPES FOR STATIC FUNCTIONS                        */
/* ************************************************************************* */



/* ************************************************************************* */
/* PROTOTYPE   : void add_new_raw_arg(cg_context_t *ctxt,plist_t *i_list,    */
/*                                    char *arg_text)                        */
/*                                                                           */
/* ARGUMENTS   : "ctxt" points to a valid context, "i_list" is the current   */
/*               parse list and "arg_text" is the textual representation of  */
/*               the argument.                                               */ 
/*                                                                           */
/* FUNCTION    : Adds a raw argument to the current instruction. The raw     */
/*               argument is a textual representation of a token. This is    */
/*               used to reconstruct the instruction.                        */
/*                                                                           */
/* RETURN VALUE: None                                                        */
/* ************************************************************************* */
void add_new_raw_arg(cg_context_t *ctxt,plist_t *i_list,char *arg_text)
{
  inst_t *temp_inst;
  arg_t *temp_arg;
  dyn_stack_t *temp_stack;
  assert((i_list != NULL) && (arg_text != NULL));
  temp_inst = get_current_inst(i_list);
  if (temp_inst == NULL) {
    iopasm_error(NULL,NULL,ERR_SYNTAX_ERROR,
		 "Trying to add empty-expression to non-existing inst.");
  }
  temp_arg = get_current_arg(temp_inst);

 
  
  if (temp_arg == NULL) {
    temp_stack = create_stack(delete_arg,
			      load_arg_object,
			      save_arg_object);
    assert(temp_stack != NULL);
    
    temp_arg = create_arg_stack(temp_stack);
    assert(temp_arg != NULL);
    add_arg(i_list,temp_arg);
  }

  add_raw_arg(temp_arg,arg_text);
}

/* ************************************************************************* */
/* PROTOTYPE   : char *duplicate_string(char *string);                       */
/*                                                                           */
/* ARGUMENTS   : "string" is a valid text string to duplicate.               */
/*                                                                           */
/* FUNCTION    : creates a copy of a string and converts it to lowercase     */
/*               characters.                                                 */
/*                                                                           */
/* RETURN VALUE: pointer to a verbatim lowercase only copy of input.         */
/* ************************************************************************* */
char *duplicate_string(char *string)
{
  int i;
  char *new_string;
  new_string = (char *) strdup(string);
  i = 0;
  while(new_string[i] != 0) {
    new_string[i] = (char) tolower((int) new_string[i]);
    i++;
  }
  
  return new_string;
}

/* ************************************************************************* */
/* PROTOTYPE   : void find_fw_reference(cg_context_t *ctxt,inst_t *inst,     */
/*                                      arg_t *arg)                          */
/*                                                                           */
/* ARGUMENTS   : "ctxt" points to a valid context. "inst" is the instruction */
/*               from where the forward search is to begin and "arg" is a    */
/*               valid pointer to the argument containing the forward ref.   */
/*               to search for.                                              */
/*                                                                           */
/* FUNCTION    : Search forward for a local label                            */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void find_fw_reference(cg_context_t *ctxt,inst_t *inst,arg_t *arg)
{
  array_t *labels;
  inst_t *temp_inst;
  inst_t *next_inst;
  char *arg_val,*inst_val;
  int i;

  assert((ctxt != NULL) && (inst != NULL) && (arg != NULL));
  
  labels = cg_get_label_array(ctxt);
  temp_inst = get_next_inst(inst);
  
  while(temp_inst != NULL) {
 
    if (get_inst_type(temp_inst) == INST_LABEL) {
      next_inst = get_next_inst(temp_inst);
	if (get_inst_type(next_inst) == INST_STATE_BEGIN) {
	  while (get_inst_type(next_inst) != INST_STATE_END) {
	    next_inst = get_next_inst(next_inst);
	    if (next_inst == NULL) {
	      ;
	      iopasm_error(ctxt,temp_inst,ERR_UNDEFINED_FW_LABEL,"");
 	    } /* End of if (next_inst == NULL) */
	  } /* End of: while (get_inst_type(next_inst) != INST_STATE_END)) */

	  temp_inst = next_inst;
	  goto run_again;
	}  /* End of if (get_inst_type(next_inst) == INST_STATE_BEGIN) */
      	
      else { /* We have a candidate to use */
	arg_val = get_arg_val(arg);
	inst_val = get_inst(temp_inst);
	i = 0;

	while(inst_val[i] != '_') { 
	  i++; 
	  if (i == strlen(inst_val)) break; /* Break out of loop */
	}

	i = (i > strlen(arg_val)) ? i: strlen(arg_val);

	if (!strncmp(arg_val,inst_val,i)) { /* Match found */
	  IOPASM_FREE(arg_val);
	  arg_val = (char *) strdup(inst_val);
	  assert(arg_val != NULL);
	  set_arg_val(arg,arg_val,strlen(arg_val));
	    arg_val = get_raw_arg(arg);
	    i = 0;
	    while(i<(strlen(arg_val) - 1)) {
	      arg_val[i] = arg_val[i+1];
	      i++;
	    }
	    arg_val[i] = 'f'; 
	    set_arg_type(arg,ARG_LABEL);
	  goto label_found;
	  } /* End of if (!strncmp(arg_val,inst_val,strlen(arg_val))) */

	/* Removing the line below fixes TR #46 */
	/* temp_inst = get_next_inst(temp_inst); */
	} /* End of !if (get_inst_type(temp_inst) == INST_STATE_BEGIN) */
    } /* End of if (get_inst_type(temp_inst) == INST_LABEL) */

  run_again:
    temp_inst = get_next_inst(temp_inst);
  } /* End of while(temp_inst != NULL) */
 label_found: /* We found it so we just return */
  return ;
}

/* ************************************************************************* */
/* PROTOTYPE   : void find_bw_reference(cg_context_t *ctxt,inst_t *inst,     */
/*                                      arg_t *arg);                         */
/*                                                                           */
/* ARGUMENTS   : "ctxt" points to a valid context. "inst" is the instruction */
/*               from where the forward search is to begin and "arg" is a    */
/*               valid pointer to the argument containing the forward ref.   */
/*               to search for.                                              */
/*                                                                           */
/* FUNCTION    : Search backward for a local label                           */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void find_bw_reference(cg_context_t *ctxt,inst_t *inst,arg_t *arg)
{
  char *arg_val,*inst_val;
  inst_t *temp_inst;
  int inst_type;

  assert((ctxt != NULL) && (inst != NULL) && (arg != NULL));
  
  arg_val = get_arg_val(arg);
  
  temp_inst = get_prev_inst(inst);
  if (temp_inst == NULL) {
    iopasm_error(ctxt,NULL,ERR_UNDEFINED_BW_LABEL,"(Label is %s)",arg_val);
  } /* End of if (temp_inst == NULL) */
  while(temp_inst != NULL) {
    inst_type = get_inst_type(temp_inst);


    if (inst_type == INST_STATE_END) {

      while(inst_type != INST_LABEL) {
	temp_inst = get_prev_inst(temp_inst);

	if (temp_inst == NULL) {
	  iopasm_error(ctxt,inst,ERR_UNDEFINED_BW_LABEL,"(Label is %s)",arg_val);
	    } /* End of if (temp_inst == NULL) */
	
	else {
	  inst_type = get_inst_type(temp_inst);
	}
      } /* End of while(inst_Type != INST_LABEL) */
    } /* End of if (inst_type == INST_STATE_END) */
    
    else {
      if (inst_type == INST_LABEL) { /* We have a possible match */
	inst_val = get_inst(temp_inst);
	if (!strncmp(arg_val,inst_val,strlen(arg_val))) {
	  IOPASM_FREE(arg_val);
	  arg_val = (char *)strdup(inst_val);
	  assert(arg_val != NULL);
	  set_arg_val(arg,arg_val,strlen(arg_val));
	  set_raw_arg(arg,arg_val);
	  set_arg_type(arg,ARG_LABEL);
	  goto end;
	}
      }
      /* Do nothing if not a label */
    } /* End of !if (inst_type == INST_STATE_END) */

    temp_inst = get_prev_inst(temp_inst);
  } /* End of while(temp_inst != NULL) */
 end: 
  return ;

}

/* ************************************************************************* */
/* PROTOTYPE   : void resolver_iterator(void *data,va_list *c_args);         */
/*                                                                           */
/* ARGUMENTS   : data is the pointer to the data for which the resolver      */
/*               function (this) is to be run. This function is called from  */
/*               within iterators and should usually be called like this:    */
/*               resolver_iterator(arg_t *, cg_context_t *, inst_t *)        */
/*                                                                           */
/* FUNCTION    : Search for local labels in argument stacks                  */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void resolver_iterator(void *data,va_list *c_args) 
{
  arg_t *the_arg;
  cg_context_t *ctxt;
  inst_t *inst;
  va_list ap;
  va_copy(ap,*c_args);
  ctxt = va_arg(ap,cg_context_t *);
  inst = va_arg(ap,inst_t *);
  the_arg = (arg_t *) data;

  if (get_arg_type(the_arg) == ARG_FW_LOCAL_LABEL) {
    find_fw_reference(ctxt,inst,the_arg);
  }
  else if (get_arg_type(the_arg) == ARG_BW_LOCAL_LABEL) {
    find_bw_reference(ctxt,inst,the_arg);
  }
  else { }
  va_end(ap);
}


/* ************************************************************************* */
/* PROTOTYPE   : void resolve_labels(cg_context_t *ctxt, plist_t *a_list);   */
/*                                                                           */
/* ARGUMENTS   : "ctxt" points to a valid context and "a_list" points to an  */
/*               instruction list.                                           */
/*                                                                           */
/* FUNCTION    : Front-end function for local label mangling purposes.       */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void resolve_labels(cg_context_t *ctxt, plist_t *a_list)
{
  inst_t *temp_inst;
  inst_t *next_inst;
  array_t *labels;
  arg_t *arg;
  dyn_stack_t *arg_stack;

  assert((ctxt != NULL) && (a_list != NULL));
  next_inst = get_first_inst(a_list);
  labels = cg_get_label_array(ctxt);
  
  while (next_inst != NULL) {
    
    switch(get_inst_type(next_inst)) {
      
    case INST_SEQ: /* Go through all arguments and look for a temporary label; */
      arg  = get_first_arg(next_inst);

      while(arg != NULL) {

	switch(get_arg_type(arg)) {
	  
	case ARG_BW_LOCAL_LABEL:
	  find_bw_reference(ctxt,next_inst,arg);
	  break;
	case ARG_FW_LOCAL_LABEL:
	  find_fw_reference(ctxt,next_inst,arg);
	  break;

	case ARG_STACK:
	  arg_stack = get_arg_stack(arg);
	  stack_iterator(arg_stack,resolver_iterator,ctxt,next_inst);
	}
	arg = get_next_arg(arg);

      } /* End of while(arg != NULL) */
      break;


    case INST_STATE_BEGIN: /* Find end of state */
      temp_inst = get_next_inst(next_inst);
      while((temp_inst != NULL) && (get_inst_type(temp_inst) != INST_STATE_END)) {
	temp_inst = get_next_inst(temp_inst);
      }
      next_inst = temp_inst;
      break;

    } /* End of switch(get_inst_type(next_inst)) */
    
    next_inst = get_next_inst(next_inst);
  } /* End of while (next_inst != NULL) */
}



/* ************************************************************************* */
/* PROTOTYPE   : int add_instruction(plist_t *a_list,char *i_text,           */
/*                                   int i_type,int size,int i_flags);       */
/*                                                                           */
/* ARGUMENTS   : "a_list" points to the list to which the instruction is add-*/
/*                ed. "i_text" is the instruction (e.g "mov"), i_type the    */
/*                type of instruction, size is the size in bytes and i_flags */
/*                are special flags (eg. relocation flags) used by code ge-  */
/*                nerators.                                                  */
/*                                                                           */
/* FUNCTION    : Front end for adding a new instruction of a specified type  */
/*               to a specific instruction list.                             */
/*                                                                           */
/* RETURN VALUE: Number of instructions in the instruction list after the    */
/*               the new instruction has been added.                         */
/* ************************************************************************* */
int add_instruction(plist_t *a_list,char *i_text,int i_type,int size,
		    int i_flags)
{
  inst_t *temp_inst;
  char *temp_string=NULL;

  assert(a_list != NULL);

  /* We must check that i_text != NULL as i_text might very well be null
   *  in certain cases eg. a timer instruction or similar */

  if (i_text != NULL) {
    temp_string = (char *) duplicate_string(i_text);  
    assert(temp_string != NULL);
  }
    
  temp_inst = create_inst(temp_string,i_type,size,i_flags);
  assert(temp_inst != NULL);

  set_inst_size(temp_inst,size);
  add_inst(a_list,temp_inst);
  IOPASM_FREE(temp_string);
  return get_list_count(a_list);
}


/* ************************************************************************* */
/* PROTOTYPE   : void add_label(cg_context_t *ctxt,plist_t *i_list,          */
/*                              char *label,int scope);                      */
/*                                                                           */
/* ARGUMENTS   : "ctxt" is the current code-generation context. i_list is    */
/*               the current instruction list. "label" is the name of the    */
/*               label and "scope" defines the scope of the label.           */
/*                                                                           */
/* FUNCTION    : Add a label to the current label database and check         */
/*               that it is not already defined.                             */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void add_label(cg_context_t *ctxt,plist_t *i_list,char *label,int scope)
{
  char *temp_string;
  label_t *temp_label;
  int i;
  array_t *label_array;
  inst_t *temp_inst;

  assert((ctxt != NULL) && (label != NULL));
  label_array = cg_get_label_array(ctxt);

  i = 0;
  while ((!isspace(label[i])) && (label[i] != 0)) { 
    i++;
  };

  label[i] = 0;
  
  temp_string = (char *) duplicate_string(label);
  temp_label = create_label(temp_string,0,scope);
  if (find_array_elem(label_array,(void *)temp_label) != NULL) {
    temp_inst = get_current_inst(i_list);
    iopasm_error(ctxt,temp_inst,ERR_DUPLICATE_LABEL,
		 "(Duplicate label is: %s)",temp_string);
    exit(1);
  }
  
  else {
    add_array_elem(label_array,temp_label);
    temp_string = (char *)duplicate_string(label);
    
    temp_inst = create_inst(temp_string,INST_LABEL,
			    0,RF_RELOCATE);


    assert(temp_inst != NULL);
    add_inst(i_list,temp_inst);
  }
}



/* ************************************************************************* */
/* PROTOTYPE   : void push_arg(plist_t *i_list,char *text,int type);         */
/*                                                                           */
/* ARGUMENTS   : "i_list" points to a valid instruction list. "text" is the  */
/*               textual representation of the argument to push and type is  */
/*               the type of argument                                        */
/*                                                                           */
/* FUNCTION    : Push an argument onto an argument stack.                    */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void push_arg(plist_t *i_list,char *text,int type)
{
  arg_t *tmp_arg,*real_arg;
  dyn_stack_t *the_stack;
  inst_t *inst;
  char *tmp_string;

  tmp_string = duplicate_string(text);
  assert((i_list != NULL) && (tmp_string != NULL));

  inst = get_current_inst(i_list);
  tmp_arg = get_first_arg(inst);

  if (tmp_arg == NULL) {
    the_stack = create_stack(delete_arg,
			     load_arg_object,
			     save_arg_object);
    assert(the_stack != NULL);
    
    tmp_arg = create_arg_stack(the_stack);
    assert(tmp_arg != NULL);
    
    real_arg = create_arg(tmp_string,strlen(tmp_string),type);
    assert(real_arg != NULL);
  
    stack_push(the_stack,(void *)real_arg);
    add_arg(i_list,tmp_arg);
  }

  else {
    while (get_next_arg(tmp_arg) != NULL) {
      tmp_arg = get_next_arg(tmp_arg);
    }

    if (get_arg_type(tmp_arg) != ARG_STACK) {
      the_stack = create_stack(delete_arg,
			       load_arg_object,
			       save_arg_object);
      assert(the_stack != NULL);

      tmp_arg = create_arg_stack(the_stack);
      assert(tmp_arg != NULL);
      add_arg(i_list,tmp_arg);

    }

    else {
      the_stack = get_arg_stack(tmp_arg);
    }
    
    real_arg = create_arg(tmp_string,strlen(tmp_string),type);
    assert(real_arg != NULL);
    stack_push(the_stack,(void *) real_arg);
  } /* End of !if(tmp_arg == NULL) */
}

/* ************************************************************************* */
/* PROTOTYPE   : void add_argument(cg_context_t *ctxt,plist_t *a_list,       */
/*                                 char *arg_text,int arg_type);             */
/*                                                                           */
/* ARGUMENTS   : "ctxt" is the current code generation context. "a_list" is  */
/*               the current instruction list. "arg_text" is the textual     */
/*               representation of the argument to be added and "arg_type"   */
/*               is the type.                                                */
/*                                                                           */
/* FUNCTION    : Adds an argument to the currently selected instruction in   */
/*               "a_list"                                                    */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void add_argument(cg_context_t *ctxt,plist_t *a_list,char *arg_text,
		  int arg_type)
{
  arg_t *temp_arg;
  int s_len;


  assert(a_list != NULL);
  s_len = (arg_text == NULL) ? 0: strlen(arg_text);

  temp_arg = create_arg(arg_text,s_len,arg_type);
  assert(temp_arg != NULL);
  add_arg(a_list,temp_arg);
}
