/* ************************************************************************* */
/* stack_eval.c: Evaluate arithmetric expressions stored in stacks in a      */
/*               machine independent manner.                                 */
/*                                                                           */
/*    Copyright 2002-2006 Axis Communications AB.                            */
/*                                                                           */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 59 Temple Place - Suite 330,                         */
/*    Boston, MA 02111-1307, USA.                                            */
/*                                                                           */
/*                                                                           */
/*    Send bug reports and inquiries to Johan.Jorgensen@axis.com             */
/*                                                                           */
/*   Date      Description                                          Sign.    */
/*  =======================================================================  */
/*  2003-09-05 Module implemented                                    JohanJ  */
/*  2004-06-19 Now uses IOPASM_MALLOC and IOPASM_FREE macros         JohanJ  */
/* ************************************************************************* */


/* ************************************************************************* */
/*                      INCLUDES USED BY THIS MODULE                         */
/* ************************************************************************* */
#include <assert.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cg_fe.h"
#include "dyn_array.h"
#include "dyn_stack.h"
#include "debug.h"
#include "iopasm_err.h"
#include "iopasm_global.h"
#include "labels.h"
#include "p_list.h"

/* ************************************************************************* */
/*                     DEFINITIONS LOCAL TO THIS MODULE                      */
/* ************************************************************************* */


/* ************************************************************************* */
/*                       TYPES DEFINED IN THIS MODULE                        */
/* ************************************************************************* */


/* ************************************************************************* */
/*                              CONSTANT VARIABLES                           */
/* ************************************************************************* */



/* ************************************************************************* */
/*                    PROTOTYPES FOR STATIC FUNCTIONS                        */
/* ************************************************************************* */
/* static void label_counter(void *val,void *argument); */
static unsigned int eval_arch_stack(cg_context_t *ctxt,inst_t *inst,
				    int inum,dyn_stack_t *arg_stack,
				    int *meta_data,int arg_no,int pass);
static unsigned int process_stack(cg_context_t *ctxt,inst_t *inst,
				  arg_t *inst_arg,
				  dyn_stack_t *arg_stack);
int check_expression_type(dyn_stack_t *a_stack);

/* ************************************************************************* */
/* PROTOTYPE   : static void count_special_register(void *data,              */
/*                                                  va_list *c_args);        */
/*                                                                           */
/* INPUT PARM's: "data" points to the data and "c_args" is an vaarg list.    */
/*                                                                           */
/* FUNCTION    : count the number of special registers in the stack.         */
/*                                                                           */
/* RETURN VALUE: Returns the number of special registers found.              */
/* ************************************************************************* */
static void count_special_register(void *data,va_list *c_args)
{
  int *cnt;
  int arg_type;
  arg_t *arg;
  va_list  ap;

  assert ((data != NULL));
  va_copy(ap,*c_args);
  
  arg = (arg_t *) data;
  cnt = va_arg(ap,int *);
  arg_type = get_arg_type(arg);
 
  if (arg_type == ARG_SPECIAL_REG) {
    (*cnt)++;
  }
  va_end(ap);
}


/* ************************************************************************* */
/* PROTOTYPE   : static void count_normal_register(void *data,               */
/*                                                 va_list *c_args);         */
/*                                                                           */
/* INPUT PARM's: "data" points to the data and "c_args" is an vaarg list.    */
/*                                                                           */
/* FUNCTION    : Count the number of normal registers in the current stack   */
/*                                                                           */
/* RETURN VALUE: Returns the number of registers in a stack.                 */
/* ************************************************************************* */
static void count_normal_register(void *data,va_list *c_args)
{
  int *cnt;
  int arg_type;
  arg_t *arg;
  va_list ap;

  assert(data != NULL);
  va_copy(ap,*c_args);

  arg = (arg_t *) data;
  cnt = va_arg(ap,int *);

  arg_type = get_arg_type(arg);
 
  if (arg_type == ARG_NORMAL_REG) {
    (*cnt)++;
  }
  va_end(ap);
}


/* ************************************************************************* */
/* PROTOTYPE   : static void count_constants(void *data,va_list *c_args);    */
/*                                                                           */
/* INPUT PARM's: "data" points to the data and "c_args" is an vaarg list.    */
/*                                                                           */
/* FUNCTION    : count the number of constants in a stack                    */
/*                                                                           */
/* RETURN VALUE: The number of constant arguments in an argument stack       */
/* ************************************************************************* */
static void count_constants(void *data,va_list *c_args)
{
  int *cnt;
  int arg_type; 
  arg_t *arg;
  va_list ap;

  assert(data != NULL);
  va_copy(ap,*c_args);

  arg = (arg_t *) data;
  cnt = va_arg(ap,int *);
  arg_type = get_arg_type(arg);

  if ((arg_type == ARG_IMMEDIATE) || (arg_type == ARG_OPERATOR)) {
    (*cnt)++;
  }
  va_end(ap);
}

/* ************************************************************************* */
/* PROTOTYPE   : void count_registers(void *data,va_list *c_args);           */
/*                                                                           */
/* INPUT PARM's: */
/*                                                                           */
/* FUNCTION    : */
/*                                                                           */
/* RETURN VALUE: */
/* ************************************************************************* */
void count_registers(void *data,va_list *c_args)
{
  int *cnt;
  int arg_type; 
  arg_t *arg;
  va_list ap;

  assert(data != NULL);
  va_copy(ap,*c_args);
  
  arg = (arg_t *) data;
  cnt = va_arg(ap,int *);
  arg_type = get_arg_type(arg);

  if ((arg_type == ARG_NORMAL_REG) || (arg_type == ARG_SPECIAL_REG) || 
      (arg_type == ARG_OPERATOR)) {
    (*cnt)++;
  }
  va_end(ap);
}

  
/* ************************************************************************* */
/* PROTOTYPE   : static void count_labels(void *data,va_list *c_args);       */
/*                                                                           */
/* INPUT PARM's: */
/*                                                                           */
/* FUNCTION    : */
/*                                                                           */
/* RETURN VALUE: */
/* ************************************************************************* */
static void count_labels(void *data,va_list *c_args)
{
  int *cnt;
  int arg_type; 
  arg_t *arg;
  va_list ap;

  assert(data != NULL);
  va_copy(ap,*c_args);


  arg = (arg_t *) data;
  cnt = va_arg(ap,int *);
  arg_type = get_arg_type(arg);

  if (arg_type == ARG_LABEL) {
    (*cnt)++;
  }
  va_end(ap);
}


/* ************************************************************************* */
/* PROTOTYPE   : static void convert_arg(arg_t *stack_arg,arg_t *arg);       */
/*                                                                           */
/* INPUT PARM's: */
/*                                                                           */
/* FUNCTION    : */
/*                                                                           */
/* RETURN VALUE: */
/* ************************************************************************* */ 
static void convert_arg(arg_t *stack_arg,arg_t *arg)
{
  char *tmp_val;
  assert ((stack_arg != NULL) && (arg != NULL));
  
  tmp_val = (char *) strdup((char *)get_arg_val(arg));
  assert(tmp_val != NULL);
  

  set_arg_type(stack_arg,get_arg_type(arg));
  set_arg_val(stack_arg,tmp_val,get_arg_len(arg));
}


/* ************************************************************************* */
/* PROTOTYPE   : static unsigned int arg2label(cg_context_t *ctxt,           */
/*                                             inst_t *inst,array_t *labels, */
/*                                             arg_t *arg);                  */
/*                                                                           */
/* INPUT PARM's: "ctxt" points to the valid code generation context, "inst"  */
/*               is the instruction associated with "arg" and "labels" is    */
/*               the array of labels.                                        */
/*                                                                           */
/* FUNCTION    : Get a value of a label from an argument descriptor          */
/*                                                                           */
/* RETURN VALUE: Value of the label according to the specified ctxt          */
/* ************************************************************************* */
static unsigned int arg2label(cg_context_t *ctxt,inst_t *inst,
			      array_t *labels,arg_t *arg)
{
  label_t *temp_label,*real_label;
  char *arg_val;
  unsigned int label_val = 0;
  assert ((ctxt != NULL) && (labels != NULL) && (arg != NULL));

  arg_val = get_arg_val(arg);
  temp_label = create_label(arg_val,0,SCOPE_NO_SCOPE);
  assert (temp_label != NULL);

  real_label = (label_t *) find_array_elem(labels,(void *)temp_label);
  
  if ((real_label == NULL) && 
      (!check_option("allow-undefined-symbols",(int *)1,OPT_INTEGER)))
    { /* Cannot find label - return error and exit */
      iopasm_error(ctxt,inst,ERR_UNDEFINED_LABEL,"Label is %s\n",arg_val);
    }
 
  else if (real_label == NULL) { /* Undefined symbols allowed and no label! */
    label_val = 0;
  }
  
  else {
    label_val = get_label_value(ctxt,real_label);
  }

  delete_label(temp_label);

  return label_val;
}


/* ************************************************************************* */
/* PROTOTYPE   : arg_t *calculate_result(cg_context_t *ctxt,inst_t *inst,    */
/*                                       dyn_stack_t *tmp_stack,arg_t *arg1, */
/*                                       arg_t *arg2,arg_t *operator);       */
/*                                                                           */
/* INPUT PARM's: "ctxt" is the current context. "inst" is the instruction    */
/*               for which to the argument expression is to be calculated.   */
/*               "tmp_stack" is a temporary scratch-pad stack used to push   */
/*               intermediate values onto. "arg1" and "arg2" are the         */
/*               to the operator "operator"                                  */
/*                                                                           */
/* FUNCTION    : Calculates the result of "arg1" "operator" "arg2"           */
/*                                                                           */
/* RETURN VALUE: Argument containing the resulting value.                    */
/* ************************************************************************* */
arg_t *calculate_result(cg_context_t *ctxt,inst_t *inst,dyn_stack_t *tmp_stack,
			arg_t *arg1,arg_t *arg2,arg_t *operator)
{
  unsigned int val1, val2;
  unsigned int result = 0;
  char *op_string;
  char tmp_string[MAX_STRLEN];
  char *result_string;
  array_t *labels;
  arg_t *new_arg;
  int i;

  assert((ctxt != NULL) && (arg1 != NULL) && 
	 (arg2 != NULL) && (operator != NULL));
  labels = cg_get_label_array(ctxt);
  
  /* Extract numerical value for arg 1 */
  if (get_arg_type(arg1) == ARG_LABEL) {
       val1 = arg2label(ctxt,inst,labels,arg1);
  } /* End of if (get_arg_type(arg1) == NULL) */

  else if (get_arg_type(arg1) == ARG_CURRENT_ADDR) {
    val1 = get_inst_addr(inst);
  }

  else { 
    val1 = strtoul(get_arg_val(arg1),(char **) NULL, 10);
  } /* end of !if (get_arg_type(arg1) == ARG_LABEL) */


  /* Extract numerical value for arg 2 */
  if (get_arg_type(arg2) == ARG_LABEL) {
    val2 = arg2label(ctxt,inst,labels,arg2);
  } /* End of if (get_arg_type(arg2) == ARG_LABEL) */

  else if (get_arg_type(arg2) == ARG_CURRENT_ADDR) {
    val2 = get_inst_addr(inst);
  }

  else {
    val2 = strtoul(get_arg_val(arg2),(char **) NULL, 10); 
  } /* End of !if (get_arg_type(arg2) == ARG_LABEL) */

  op_string = get_arg_val(operator);

  switch(op_string[0]) {
  case '~':
    if (get_arg_type(arg2) != ARG_OPERATOR) {
      new_arg = copy_arg(arg1);
      assert(new_arg != NULL);
      stack_push(tmp_stack,new_arg);
      result = ~val2;
    }

    else {
      result = ~val1;
    }
    break;

  case '+':
    result = val1 + val2;
    break;

  case '-':
    result = val1 - val2;
    break;

  case '*':
    result = val1 * val2;
    break;

  case '/':
    result = val1 / val2;
    break;
    
  case '<':
    if (strlen(op_string) == 1) {
      result = val1 < val2;
    }
    else {
      switch(op_string[1]) {
      case '<':
	result = val1 << val2;
	break;
	
      case '=':
	result = val1 <= val2;
	break;
      }
    }
    break;
    
  case '>':
    if (strlen(op_string) == 1) {
      result = val1 > val2;
    }
    else {
      switch(op_string[1]) {
      case '>':
	result = val1 >> val2;
	break;
	
      case '=':
	result = val1 >= val2;
	break;
      }
    }
    break;

  case '|':
    if (strlen(op_string) == 1) {
      result = val1 | val2;
    }
    else {
      result = val1 || val2;
    }
    break;

  case '&':
    if (strlen(op_string) == 1) {
      result = val1 & val2;
    }
    else {
      result = val1 && val2;
    }
    break;

  case '^':
    result = val1 ^ val2;
    break;

  case '=':
    result = (val1 == val2);
    break;

  case '!':
    result = (val1 != val2);
    break;
  }
  

  memset(tmp_string,0,MAX_STRLEN);
  i = snprintf(tmp_string,MAX_STRLEN,"%u",result);
  
  result_string = (char *) IOPASM_MALLOC(sizeof(char ) * (i + 1));
  assert(result_string != NULL);

  memset(result_string,0,(i + 1));
  strncpy(result_string,tmp_string,i);
  
  return  create_arg(result_string,strlen(result_string),ARG_IMMEDIATE);
}
  
      
/* ************************************************************************* */
/* PROTOTYPE   : static unsigned int process_stack(cg_context_t *ctxt,       */
/*                                                 inst_t *inst,             */
/* 				                   arg_t *inst_arg,          */
/* 				                   dyn_stack_t *arg_stack);  */
/*                                                                           */
/* INPUT PARM's: "ctxt" is the current context. "inst" is the current        */
/*               instruction. "inst_arg" is the instruction argument and     */
/*               "arg_stack" is the stack to process.                        */
/*                                                                           */
/* FUNCTION    : Helper function used to convert a stack to an unsigned      */
/*               int representing the op-code for the argument "inst_arg"/   */
/*               "arg_stack"                                                 */
/*                                                                           */
/* RETURN VALUE: unsigned integer representing the resulting op-code         */
/* ************************************************************************* */
static unsigned int process_stack(cg_context_t *ctxt,inst_t *inst,
				  arg_t *inst_arg,
				  dyn_stack_t *arg_stack)
{
  arg_t *arg1,*arg2,*operator;
  dyn_stack_t *tmp_stack;
  arg_t *result = NULL;
  arg_t *tmp_arg;
  char *str_result;
  unsigned int n_result;

  assert ((inst_arg != NULL) && (arg_stack != NULL));

  tmp_stack = create_stack(delete_arg,
			   load_arg_object,
			   save_arg_object);
  assert(tmp_stack != NULL);
  
  while(get_stack_depth(arg_stack) > 1) {
    
    arg1 = (arg_t *) stack_pop(arg_stack);
    arg2 = (arg_t *) stack_pop(arg_stack);

    if (get_arg_type(arg2) != ARG_OPERATOR) { 
      operator = (arg_t *) stack_pop(arg_stack);
    }

    else {
      operator = arg2;
    }

    while(get_arg_type(operator) != ARG_OPERATOR) {
      stack_push(tmp_stack,(void *)arg1);
      arg1 = arg2;
      arg2 = operator;
      operator = stack_pop(arg_stack);
    } /* End of while(get_arg_type(... */
    
    result = calculate_result(ctxt,inst,tmp_stack,arg1,arg2,operator);
    assert(result != NULL);

    assert(result != NULL);
    
    stack_push(tmp_stack,result);
    delete_arg(arg1);

    if (arg2 == operator) {
      delete_arg(arg2);
    } /* End of if (arg2 == operator) */

    else {
      delete_arg(arg2);
      delete_arg(operator);
    }  /* End of !if (arg2 == operator) */

    while(get_stack_depth(tmp_stack)>0) {
      tmp_arg = (arg_t *) stack_pop(tmp_stack);
      stack_push(arg_stack,tmp_arg);
    }
  } /* End of while(get_stack_depth(... */
  
  /* The stack is now empty, so let us remove it!"! */
  arg1 = (arg_t *) stack_pop(arg_stack);
  convert_arg(inst_arg,arg1);
  delete_stack(arg_stack);
  delete_stack(tmp_stack);
  str_result = get_arg_val(result);
  n_result = strtoul(str_result,NULL,10);
  return n_result;
}

/* ************************************************************************* */
/* PROTOTYPE   : int convert_stack_arg(cg_context_t *ctxt,inst_t *inst,      */
/*                                     arg_t *arg,int pass);                 */
/*                                                                           */
/* INPUT PARM's: "ctxt" is the current code generation context. "inst" is    */
/*               the current instruction. "arg" is the argument of "inst"    */
/*               containing the stack and "pass" is the assembler pass.      */
/*                                                                           */
/* FUNCTION    : Front-end of the stack-handling mechanism used by backends  */
/*               to convert an argument stack into a regular argument.       */
/*                                                                           */
/* RETURN VALUE: Type of stack.                                             */
/* ************************************************************************* */
int convert_stack_arg(cg_context_t *ctxt,inst_t *inst, arg_t *arg,int pass)
{

  int ret_type;
  dyn_stack_t *the_stack;

  assert((ctxt != NULL) && (arg != NULL));
  
  if (get_arg_type(arg) != ARG_STACK) {
    return get_arg_type(arg);
  }
  
  else {
    the_stack = get_arg_stack(arg);
  }

  /* Now we are ready to convert the stack */

  ret_type = check_expression_type(the_stack);

  if (((ret_type == ARG_LABEL) ||(ret_type == ARG_CURRENT_ADDR)) && (pass == FINAL_PASS)) {
    ret_type = ARG_IMMEDIATE;
  }
  
  switch (ret_type) {
  case ARG_LABEL:
  case ARG_IMMEDIATE:
    invert_stack(the_stack);
    process_stack(ctxt,inst,arg,the_stack);
    break;
  }

  return ret_type;
}



/* ************************************************************************* */
/* PROTOTYPE   : unsigned int minimize_stacks(cg_context_t *ctxt,            */
/*                                            inst_t *inst,int pass);        */
/*                                                                           */
/* INPUT PARM's: "ctxt" is the current code-generation context. "inst" is    */
/*               the current instruction and pass represents the current     */
/*               assembler pass.                                             */
/*                                                                           */
/* FUNCTION    : Minimize (remove all) stacks that can be removed at the     */
/*               intermediate pass.                                          */
/*                                                                           */
/* RETURN VALUE: None.                                                       */
/* ************************************************************************* */
void minimize_stacks(cg_context_t *ctxt,inst_t *inst,int pass)
{
  dyn_stack_t *tmp_stack;
  arg_t *arg,*tmp_arg;

  assert((ctxt != NULL) && (inst != NULL));

  arg = get_first_arg(inst);

  while (arg != NULL) {
    if (get_arg_type(arg) == ARG_STACK) {
      tmp_stack = get_arg_stack(arg);

      if (get_stack_depth(tmp_stack) == 1) {
	tmp_arg = stack_pop(tmp_stack);
	convert_arg(arg,tmp_arg);
      }
    }
    arg = get_next_arg(arg);
  }
}


/* ************************************************************************* */
/* PROTOTYPE   : unsigned int eval_arg_stack(cg_context_t *ctxt,             */
/*                                           inst_t *inst,int inum,          */
/* 			                     arg_t *arg_stack,               */
/*                                           int *meta_data,int arg_no,      */
/*                                           int pass);                      */
/*                                                                           */
/* INPUT PARM's: "ctxt" is the current code-generation context. "inst" is    */
/*               the current instruction with the number "inum". "arg_stack" */
/*               "meta_data" is a bitfield used to store information about   */
/*               register expressions (used by code generator). "arg_no" is  */
/*               the number of the argument relative to the instruction and  */
/*               "pass" is the assembler pass.                               */
/*                                                                           */
/* FUNCTION    : Front-end function used by code generators to calculate     */
/*               the equivalent value of a stack.                            */
/*                                                                           */
/* RETURN VALUE: Op-code specific part of the stack                          */
/* ************************************************************************* */
unsigned int eval_arg_stack(cg_context_t *ctxt,inst_t *inst,int inum,
			    arg_t *arg_stack,int *meta_data,int arg_no,int pass)
{
  dyn_stack_t *the_stack;
  int imm_count;

  assert((ctxt != NULL) && (arg_stack != NULL));

  if (get_arg_type(arg_stack) != ARG_STACK) {
    return 0;
  }

  the_stack = get_arg_stack(arg_stack);
  imm_count = 0;
  stack_iterator(the_stack,count_constants,&imm_count);
  
  /* We only have constants in this stack - fix it! */
  if (get_stack_depth(the_stack) == imm_count) { /* Evaluate stack */
    invert_stack(the_stack);
    return process_stack(ctxt,inst,arg_stack,the_stack);
  } /* End of if (get_stack_depth(... */

  /* Prepare and process a stack with labels in it! */
  imm_count = 0;
  stack_iterator(the_stack,count_labels,&imm_count);
  
  if ((imm_count > 0)  && (pass == INTERMEDIATE_PASS )) {
    set_reloc_flag(inst,RF_RELOCATE);
  } /* End of if ((imm_count > 0)  &&(pass == INTERMEDIATE_PASS )) { */
    
  if ((imm_count >0) && (pass == FINAL_PASS)) {
    /* Process a stack with labels here */
    invert_stack(the_stack);
    return process_stack(ctxt,inst,arg_stack,the_stack);
  } /* End of else if ((imm_count != NULL) pass == FINAL_PASS) { */
  
  if ((imm_count == 0)) {
    imm_count = 0;
    stack_iterator(the_stack,count_registers,&imm_count);
    if (imm_count >0) {
      invert_stack(the_stack);
      return eval_arch_stack(ctxt,inst,inum,the_stack,meta_data,arg_no,pass); 
    }
  } /* End of !if ((imm_count == 0) */

  return 0;
}


/* ************************************************************************* */
/* PROTOTYPE   : static unsigned int eval_arch_stack(cg_context_t *ctxt,     */
/*                                                   inst_t *inst,int inum,  */
/*                                                   dyn_stack_t *arg_stack, */
/*				    int *meta_data,int arg_no,int pass);     */
/*                                                                           */
/* INPUT PARM's: "ctxt" is a valid context. "inst" is the instruction and    */
/*               is the corresponding instruction number. "arg_stack" is the */
/*               stack of arguments to evaluate. "meta_data" contains meta-  */
/*               information about the evaluation of the stack. "arg_no" is  */
/*               the argument number relative to the instruction. "pass" is  */
/*               the assembly pass.                                          */
/*                                                                           */
/* FUNCTION    : Front-end function called to calculate the numerical value  */
/*               of a stack.                                                 */
/*                                                                           */
/* RETURN VALUE: Op-code representation of the current stack.                */
/* ************************************************************************* */
static unsigned int eval_arch_stack(cg_context_t *ctxt,inst_t *inst,int inum,
				    dyn_stack_t *arg_stack,
				    int *meta_data,int arg_no,int pass)
{
  
  unsigned int tmp_val;
  arg_t *operator;
  arg_t *arg1,*arg2;
  char *operator_type;
  
  assert ((ctxt != NULL) && (arg_stack != NULL) && (meta_data != NULL));

  arg1 = (arg_t *) stack_pop(arg_stack);
  arg2 = (arg_t *) stack_pop(arg_stack);
  operator = (arg_t *) stack_pop(arg_stack);

  operator_type = get_arg_val(operator);
  tmp_val = 0;
  cg_eval_operator(ctxt,operator_type,meta_data,sizeof(int),&tmp_val,sizeof(int),
     inst,inum,arg1,arg2,arg_no);
  return tmp_val;
}


/* ************************************************************************* */
/* PROTOTYPE   : int check_expression_type(dyn_stack_t *a_stack);            */
/*                                                                           */
/* INPUT PARM's: a_stack points to a valid dynamic stack.                    */
/*                                                                           */
/* FUNCTION    : Check the type that the expression evaluates to.            */
/*                                                                           */
/* RETURN VALUE: Expression type (Register, label etc.)                      */
/* ************************************************************************* */
int check_expression_type(dyn_stack_t *a_stack)
{
  int reg_count,imm_count;
  int label_count;

  reg_count = 0;
  imm_count = 0;
  label_count = 0;

  stack_iterator(a_stack,count_normal_register,&reg_count);
  stack_iterator(a_stack,count_constants,&imm_count);
  stack_iterator(a_stack,count_labels,&label_count);

  if (reg_count > 0) {
    return ARG_NORMAL_REG;
  }

  else if (label_count > 0) {
    return ARG_LABEL;
  }

  else if (imm_count > 0) {
    return ARG_IMMEDIATE;
  }


  else {
    return ARG_NO_ARG;
  }

}
